
;****************************************************************************
;                                873RS11.ASM
;             Utilisation de la rs232 en moniteur sous IT
;             Mise en place du module can sous IT timer 0
;             Mise en place du module pwm sous IT timer 0
;             decalage pour les 3 groupes de sortie:
;             - out
;             - mat-out
;             - pwm
;              9.passage a 50 Hz
;             10.support du capteur de courant       
;             11.amelioration de la liaison serie
;             12.rajout d'un servo (pour plaque tournante)
;             13.modification du moniteur (maintenant adressable (8 adresses))
;****************************************************************************
;
; 0 sync
; 1 reply 83
; 2 set   current adr
; 3 write data at current adr
; 4 read  data at current adr
; 5 increment current adr and write data at current adr 
; 6 increment current adr and read  data at current adr
; 7 increment current adr and read  data at current adr and clear current register
; 8-15 total transfere 
; 16-255 only the 4 LSB are processed
;
; rx    0    1    2    3    4    5    6     7     8
; tx         83             data      +data +data
;
; rx2             adr  data      +data            data
; tx2                                             data
;
; Le ping (1) permet egalement de choisir la carte 0x01 0x11 0x21 0x31 0x41 0x51 0x61 0x71
;
; registres:
; can0:0x2a can1:0x2b can2:0x2c can3:0x2d can4:0x2e
; pwm0:0x33 pwm1:0x34 pwm2:0x35 pwm3:0x36 sens:0x37


        LIST    p=16CXX ; target processor
                        ; car mon assembleur ne connait pas le 872




;=============================================================================
;====                              constantes                             ====
;=============================================================================


;*** adresses des registres standards (et bank) ***
CARDADR EQU 3 ;card adresse (0-7)

;-----------------------------------------------------------------------------

;*** adresses des registres standards (et bank) ***
INDF    EQU     0x00   ;0123
TMR0    EQU     0x01   ;0 2
PCL     EQU     0x02   ;0123
STATUS  EQU     0x03   ;0123
FSR     EQU     0x04   ;0123
PORTA   EQU     0x05   ;0
PORTB   EQU     0x06   ;0 2
PORTC   EQU     0x07   ;0
PCLATH  EQU     0x0A   ;0123
CARRY   EQU     0      ;de STATUS
ZERO    EQU     2      ;de STATUS
RP0     EQU     5      ;de STATUS
RP1     EQU     6      ;de STATUS
IRP     EQU     7      ;de STATUS

TMR1L   EQU     0x0e
TMR1H   EQU     0x0f
T1CON   EQU     0x10
CCPR1L  EQU     0x15
CCPR1H  EQU     0x16
CCP1CON EQU     0x17
CCPR2L  EQU     0x1b
CCPR2H  EQU     0x1c
CCP2CON EQU     0x1d
TMR1ON  EQU     0      ;de T1CON

OPTREG  EQU     0x01   ; 1 3
TRISA   EQU     0x05   ; 1
TRISB   EQU     0x06   ; 1 3
TRISC   EQU     0x07   ; 1 

EEDATA  EQU     0x0C   ;  2
EEADR   EQU     0x0D   ;  2
EEDATH  EQU     0x0E   ;  2
EEADRH  EQU     0x0F   ;  2

EECON1  EQU     0x0C   ;   3
EECON2  EQU     0x0D   ;   3
RD      EQU     0x00   ;de EECON1
EEPGD   EQU     0x07   ;de EECON1

RCSTA   EQU     0x18   ;0
TXSTA   EQU     0x18   ; 1
TXREG   EQU     0x19   ;0
RCREG   EQU     0x1a   ;0
SPBRG   EQU     0x19   ; 1

INTCON  EQU     0x0B   ;0123
PIR1    EQU     0x0C   ;0
PIR2    EQU     0x0D   ;0
PIE1    EQU     0x0C   ; 1
PIE2    EQU     0x0D   ; 1
GIE     EQU     7      ;de INTCON
PEIE    EQU     6      ;de INTCON
T0IE    EQU     5      ;de INTCON
T0IF    EQU     2      ;de INTCON           
RCIF    EQU     5      ;de PIR1           
RCIE    EQU     5      ;de PIE1
CCP1IF  EQU     2      ;de PIR1
CCP2IF  EQU     0      ;de PIR2

ADCON0  EQU     0x1f   ;0
ADCON1  EQU     0x1f   ; 1
ADRESH  EQU     0x1e   ;0
ADRESL  EQU     0x1e   ; 1

CLKOUT  EQU     7      ;de PORTB
LODOUT  EQU     6      ;de PORTB
DATOUT  EQU     5      ;de PORTB
CLKPWM  EQU     1      ;de PORTB
LODPWM  EQU     2      ;de PORTB
DATPWM  EQU     4      ;de PORTB
CLKMAT  EQU     4      ;de PORTC
DATMAT  EQU     5      ;de PORTC


;-----------------------------------------------------------------------------

;*** adresse des variables (bank 0) ***

pwm_r0   EQU     0x20   ;0    registres virtuels des sorties
pwm_r1   EQU     0x21   ;0
pwm_r2   EQU     0x22   ;0
pwm_r3   EQU     0x23   ;0
pwm_s03  EQU     0x24   ;0
pwm_r4   EQU     0x25   ;0
pwm_r5   EQU     0x26   ;0
pwm_r6   EQU     0x27   ;0
pwm_r7   EQU     0x28   ;0
pwm_s47  EQU     0x29   ;0
pwm_r8   EQU     0x2a   ;0
pwm_r9   EQU     0x2b   ;0
pwm_r10  EQU     0x2c   ;0
pwm_r11  EQU     0x2d   ;0
pwm_s8   EQU     0x2e   ;0
pwm_r12  EQU     0x2f   ;0
pwm_r13  EQU     0x30   ;0
pwm_r14  EQU     0x31   ;0
pwm_r15  EQU     0x32   ;0
pwm_s12  EQU     0x33   ;0
out_out3 EQU     0x34   ;0
out_out4 EQU     0x35   ;0
out_out5 EQU     0x36   ;0
led_on0  EQU     0x37   ;0
led_on1  EQU     0x38   ;0
led_on01 EQU     0x39   ;0
led_on2  EQU     0x3a   ;0
led_on3  EQU     0x3b   ;0
led_on23 EQU     0x3c   ;0
led_cl0  EQU     0x3d   ;0
led_cl1  EQU     0x3e   ;0
led_cl01 EQU     0x3f   ;0
led_cl2  EQU     0x40   ;0
led_cl3  EQU     0x41   ;0
led_cl23 EQU     0x42   ;0
led_in0  EQU     0x43   ;0
led_in1  EQU     0x44   ;0
led_in01 EQU     0x45   ;0
led_in2  EQU     0x46   ;0
led_in3  EQU     0x47   ;0
led_in23 EQU     0x48   ;0
srv_0    EQU     0x49   ;0    srv_1 en 0x5f

can_r0   EQU     0x4a   ;0    registres virtuels des entrees
can_r1   EQU     0x4b   ;0
can_r2   EQU     0x4c   ;0
can_r3   EQU     0x4d   ;0
can_r4   EQU     0x4e   ;0
det_i1   EQU     0x4f   ;0
det_i2   EQU     0x50   ;0
det_i1m  EQU     0x51   ;0
det_i2m  EQU     0x52   ;0
mat_in0  EQU     0x53   ;0
mat_in1  EQU     0x54   ;0
mat_in2  EQU     0x55   ;0
mat_in3  EQU     0x56   ;0
mat_in4  EQU     0x57   ;0
mat_in5  EQU     0x58   ;0
mat_in6  EQU     0x59   ;0
mat_in7  EQU     0x5a   ;0
mat_in8  EQU     0x5b   ;0

rs_sel   EQU     0x5c   ;0    registres de variables
ccp2f_h  EQU     0x5d   ;0
ccp2f_l  EQU     0x5e   ;0
srv_1    EQU     0x5f   ;0
W_TEMP   EQU     0x60   ;01    
S_TEMP   EQU     0x61   ;0
W_TEMP2  EQU     0x62   ;01
S_TEMP2  EQU     0x63   ;0
cpt      EQU     0x64   ;0
F_TEMP   EQU     0x65   ;0
rs_adr   EQU     0x66   ;0
rs_mode  EQU     0x67   ;0  
rs_rx    EQU     0x68   ;0
can_mode EQU     0x69   ;0
cpt2     EQU     0x6a   ;0
cpt_i    EQU     0x6b   ;0
det_i1t  EQU     0x6c   ;0
det_i2t  EQU     0x6d   ;0
led_cpt  EQU     0x6e   ;0
led_cli  EQU     0x6f   ;0
mat_tmp  EQU     0x70   ;0
proc_on  EQU     0x71   ;0
pwm_cpt  EQU     0x72   ;0
pwm_stp  EQU     0x73   ;0
pwm_wrk  EQU     0x74   ;0
out_out1 EQU     0x75   ;0
out_out2 EQU     0x76   ;0
out_pwm1 EQU     0x77   ;0
out_pwm2 EQU     0x78   ;0
out_pwm3 EQU     0x79   ;0
out_pwm4 EQU     0x7a   ;0
out_mat  EQU     0x7b   ;0

ccpr_h   EQU     0x7c   ;0
ccpf_h   EQU     0x7d   ;0
ccpf_l   EQU     0x7e   ;0
it_err   EQU     0x7f   ;0

;-----------------------------------------------------------------------------




;=============================================================================
;====                              programme                              ====
;=============================================================================


;-----------------------------------------------------------------------------

        ORG     0
        goto    init


        ORG     4
        ;btfsc   INTCON,T0IF
        goto    it_timer
        ;btfsc   PIR1,RCIF
        ;goto    it_rs232
        ;return

;-----------------------------------------------------------------------------

init
        call    init_port
        call    init_can
        call    init_pwm
        call    init_led
        call    init_srv
        call    init_rs232
        call    init_timer
        call    init_proc
	call    init_i
        call    init_it
tdf
        goto    tdf

;-----------------------------------------------------------------------------

init_proc
        movlw   0xef
        movwf   proc_on
        return

;-----------------------------------------------------------------------------

init_srv
        movlw   0x31     ;inc 4us
	movwf   T1CON
        movlw   9        ;9 clr when match (8 set when match)
        movwf   CCP1CON
        movwf   CCP2CON
        bcf     PIR1,CCP1IF
        bcf     PIR2,CCP2IF
        movlw   0x5c     ;92 (1500us) central position (avoid extremities) 
        movwf   srv_0 
        movwf   srv_1 
        return

;-----------------------------------------------------------------------------

init_i
        clrf    cpt_i
	clrf    det_i1
	clrf    det_i2
	clrf    det_i1m
	clrf    det_i2m
        return

;-----------------------------------------------------------------------------

init_it
        bcf     PIR1,RCIF      ;enable rs232 IT
        ;bsf     STATUS,RP0     ;bank 1
        ;bsf     PIE1,RCIE
        ;bcf     STATUS,RP0     ;bank 0

        bcf     INTCON,T0IF    ;enable timer0 IT
        bsf     INTCON,T0IE

        bsf     INTCON,PEIE    ;enable IT
        bsf     INTCON,GIE

        clrf    it_err

        return

;-----------------------------------------------------------------------------

init_port
        clrf    PORTB          ;initialisation des sorties a 0
        clrf    PORTA
        clrf    PORTC
        bsf     STATUS,RP0     ;initialisation des entrees et sorties
        movlw   0x08           ;PORTB en sortie (sauf B3 capteur de courant)
	movwf   TRISB          
	movlw   0x2f           ;PORTA bit4 en sortie
        movwf   TRISA
        movlw   0x8f           ;PORTC en sortie (sauf rx(pc7))
        ;movlw   0x8b          ;PORTC en sortie (sauf rx(pc7) ccp1(pc2)) (enable by software)
        ;movlw   0x8d          ;PORTC en sortie (sauf rx(pc7) ccp2(pc1)) (enable by software)
        ;movlw   0x89          ;PORTC en sortie (sauf rx(pc7) ccp1(pc2) ccp2(pc1)) (enable by software)
        movwf   TRISC
        bcf     STATUS,RP0     ;bank 0

        clrf    out_out1
        clrf    out_out2
        clrf    out_out3
        clrf    out_out4
        clrf    out_out5
        clrf    out_pwm1
        clrf    out_pwm2
        clrf    out_pwm3
        clrf    out_pwm4

        clrf    mat_in0
        clrf    mat_in1
        clrf    mat_in2
        clrf    mat_in3
        clrf    mat_in4
        clrf    mat_in5
        clrf    mat_in6
        clrf    mat_in7
        clrf    mat_in8

        return

;-----------------------------------------------------------------------------

init_can
        bsf     STATUS,RP0     ;bank 1
        clrf    ADCON1         ;RESADRH 8bits toutes entrees can en mode can
        bcf     STATUS,RP0     ;bank 0
        movlw   0x81           ;can on,Tad=4us a 8MHz,conv=48us
        movwf   ADCON0
        clrf    can_mode
        return

;-----------------------------------------------------------------------------

init_pwm
        movlw   0x40           ;64 pas
        movwf   pwm_stp
        clrf    pwm_cpt
        clrf    pwm_r0
        clrf    pwm_r1
        clrf    pwm_r2
        clrf    pwm_r3
        clrf    pwm_r4
        clrf    pwm_r5
        clrf    pwm_r6
        clrf    pwm_r7
        clrf    pwm_r8
        clrf    pwm_r9
        clrf    pwm_r10
        clrf    pwm_r11
        clrf    pwm_r12
        clrf    pwm_r13
        clrf    pwm_r14
        clrf    pwm_r15
        clrf    pwm_s03
        clrf    pwm_s47
        clrf    pwm_s8
        clrf    pwm_s12

        return

;-----------------------------------------------------------------------------

init_rs232
        clrf    rs_mode

        bsf     STATUS,RP0     ;bank 1
        movlw   0x19           ;207=0xcf=1200bauds 25=0x19=9600 (hight vit)
        movwf   SPBRG          ;a 4MHz donc vitesses a multiplier par 2 a 8MHz
        movlw   0x24           ;asynchrone,tx enable,hight vit
        movwf   TXSTA
        bcf     STATUS,RP0     ;bank 0
        movlw   0x90
        movwf   RCSTA          ;pin rs232,rx enable
        bcf     PIR1,RCIF      ;clr rx flag
        clrf    rs_sel         ;unselect card
        return

;-----------------------------------------------------------------------------
init_led
        clrf  led_on0
        clrf  led_on1
        clrf  led_on01
        clrf  led_on2
        clrf  led_on3
        clrf  led_on23
        movlw 0xff
        movwf led_cl0
        movwf led_cl1
        movwf led_cl01
        movwf led_cl2
        movwf led_cl3
        movwf led_cl23
        clrf  led_in0
        clrf  led_in1
        clrf  led_in01
        clrf  led_in2
        clrf  led_in3
        clrf  led_in23
        clrf  led_cpt  
                
        return

;-----------------------------------------------------------------------------

init_timer
        bsf     STATUS,RP0     ;bank 1
        movlw   0xc2           ;base de temp: 0=1us  1=2us 2=4us .. 7=128us
        movwf   OPTREG
        bcf     STATUS,RP0     ;bank 0
        return

;-----------------------------------------------------------------------------

it_timer
        bcf     INTCON,GIE

        movwf   W_TEMP2     ;sauvegarde W,STATUS
        swapf   STATUS,w
        clrf    STATUS
        movwf   S_TEMP2

        bcf     INTCON,T0IF ;mise a 0 du drapeau d'IT
        movlw   0xb4        ;duree=(255-XXXX)*temps_us  255- 75=180(0xb4) 75*4us =300us   600 instr
;       movlw   0x69        ;duree=(255-XXXX)*temps_us  255-150=105(0x69) 150*4us=600us  1200 instr
        movwf   TMR0

        incf    cpt,f
        btfsc   STATUS,ZERO
        incf    cpt2,f

	btfsc   cpt_i,7
	call    get_i

        btfsc   proc_on,0   ;appel des fonctions regulieres
        call    cal_pwm     
        btfsc   proc_on,1
        call    maj_pwm

        call    maj_rs232

        btfss   cpt,2       ;repartiteur de charge
        goto    cpt_0xx
cpt_1xx btfss   cpt,1
        goto    cpt_10x
cpt_11x btfss   cpt,0
        goto    cpt_6
        goto    cpt_7
cpt_10x btfss   cpt,0
        goto    cpt_4
        goto    cpt_5
cpt_0xx btfss   cpt,1
        goto    cpt_00x
cpt_01x btfss   cpt,0
        goto    cpt_2
        goto    cpt_3
cpt_00x btfss   cpt,0    
        goto    cpt_0
        goto    cpt_1


cpt_0   btfsc   proc_on,3
        call    maj_out5
        btfsc   proc_on,2
        call    cal_led
        goto    cpt_f
cpt_1   btfsc   proc_on,3
        call    maj_out4
        btfsc   proc_on,6
        call    maj_can1
        goto    cpt_f
cpt_2   btfsc   proc_on,3
        call    maj_out3
        btfsc   proc_on,7
        call    maj_sig
        goto    cpt_f
cpt_3   btfsc   proc_on,3
        call    maj_out2
        btfsc   proc_on,6
        call    maj_can2
        goto    cpt_f
cpt_4   btfsc   proc_on,3
        call    maj_out1
        goto    cpt_f
cpt_5   btfsc   proc_on,5
        call    maj_in
        goto    cpt_f
cpt_6   btfsc   proc_on,4
        call    maj_mat
        goto    cpt_f
cpt_7   call    servo_l
        goto    cpt_f
cpt_f
        call    servo_h
        btfsc   INTCON,T0IF
        incf    it_err,f
 
        swapf   S_TEMP2,w   ;restoration W,STATUS
        movwf   STATUS
        swapf   W_TEMP2,f
        swapf   W_TEMP2,w

        bsf     INTCON,GIE

        retfie

;-----------------------------------------------------------------------------

maj_sig

        btfss   cpt2,3      ;clignotement PORTA bit4 (1.62Hz)
        bcf     PORTA,4
        btfsc   cpt2,3
        bsf     PORTA,4
        return

;-----------------------------------------------------------------------------
;env 40us

maj_rs232

        btfss   PIR1,5        ;retourne si rien recu
        return

        movf    FSR,w         ;save FSR
        movwf   F_TEMP

        movf    RCREG,w       ;lire rcreg efface PIR1,5
        movwf   rs_rx

        btfsc   rs_mode,7
        goto    rs_l2y

rs_l2n  btfsc   rs_rx,3
        goto    rs_8
	btfss   rs_rx,2
        goto    rs_0xx
rs_1xx  btfss   rs_rx,1
        goto    rs_10x
rs_11x  btfss   rs_rx,0
        goto    rs_6
        goto    rs_7
rs_10x  btfss   rs_rx,0
        goto    rs_4
        goto    rs_5
rs_0xx  btfss   rs_rx,1
        goto    rs_00x
rs_01x  btfss   rs_rx,0
        goto    rs_2
        goto    rs_3
rs_00x  btfss   rs_rx,0    
        goto    rs_0
        goto    rs_1
rs_1    bcf     rs_sel,0
        swapf   rs_rx,w
        andlw   0x07
        sublw   CARDADR
        btfss   STATUS,CARRY
        goto    rs_fin
        bsf     rs_sel,0
        movlw   0x83       ;reply 0x83
        call    tx
        goto    rs_fin
rs_2    movlw   0x81       ;prepare set adr
        movwf   rs_mode
        goto    rs_fin
rs_3    movlw   0x82       ;prepare write
        movwf   rs_mode
        goto    rs_fin
rs_5    movlw   0x84       ;prepare inc write
        movwf   rs_mode
        goto    rs_fin
rs_6    btfss   rs_sel,0
        goto    rs_fin
        incf    rs_adr,f   ;inc adr and read
rs_4    btfss   rs_sel,0
        goto    rs_fin
        movf    rs_adr,w   ;read
        movwf   FSR
        movf    INDF,w
        call    tx
        goto    rs_fin
rs_7    btfss   rs_sel,0
        goto    rs_fin     
        incf    rs_adr,f 
	movf    rs_adr,w   ;inc adr + read + clear
        movwf   FSR
        movf    INDF,w
        clrf    INDF
        call    tx
	goto    rs_fin
rs_8    movlw   0x88       ;prepare total write
        movwf   rs_mode
	movlw   0x20-1
	movwf   rs_adr
rs_0    goto    rs_fin     ;sync


rs_l2y  btfsc   rs_mode,3
	goto    rs_2_8
	bcf     rs_mode,7
        btfss   rs_sel,0
        goto    rs_fin
        btfsc   rs_mode,0
        goto    rs_2_2
        btfsc   rs_mode,1
        goto    rs_2_3
        goto    rs_2_5
rs_2_2  movf    rs_rx,w    ;set adr
        movwf   rs_adr
        goto    rs_fin
rs_2_5  incf    rs_adr,f   ;inc adr and write
rs_2_3  movf    rs_adr,w   ;write
        movwf   FSR
        movf    rs_rx,w
        movwf   INDF
        goto    rs_fin

rs_2_8  incf    rs_adr,f    ;global transfere 
        movlw   0x48          ;si derniere adresse write
	subwf   rs_adr,w
        btfsc   STATUS,ZERO
	bcf     rs_mode,7     ;ne plus executer l2y   
        btfss   rs_sel,0
        goto    rs_fin
	movf    rs_adr,w   
        movwf   FSR
        movf    rs_rx,w
        movwf   INDF 
	movlw   0x4a-0x20
	addwf   FSR,f
	movlw   0xff-0x5b
	addwf   FSR,w
	btfsc   STATUS,CARRY
	goto    rs_fin        ;fin read si > a 0x5b
        movf    INDF,w
	call    tx            
	movlw   0xff-0x50     ;effacement 0x51-0x5b
	addwf   FSR,w
	btfsc   STATUS,CARRY
        clrf    INDF          ;goto    rs_fin

rs_fin
        movf    F_TEMP,w   ;restore FSR
        movwf   FSR

        return


;-----------------------------------------------------------------------------

tx      bsf     STATUS,RP0     ;bank 1
        btfss   TXSTA,1        ;attente shift register empty (1)
        goto    tx
        bcf     STATUS,RP0     ;bank 0
        movwf   TXREG
        return

;-----------------------------------------------------------------------------

;rx      btfss   PIR1,5
;        goto    rx
;        movf    RCREG,w       ;lire rcreg efface PIR1,5  
;        return

;-----------------------------------------------------------------------------

maj_can1
        movf    ADRESH,w

        btfsc   can_mode,2
        goto    can_4
        btfss   can_mode,1
        goto    can_0x
can_1x  btfss   can_mode,0
        goto    can_2
        goto    can_3
can_0x  btfss   can_mode,0
        goto    can_0
        goto    can_1

can_0   movwf   can_r0
        movlw   1         ;prochain 1
        movwf   can_mode
        movlw   0x49
        movwf   ADCON0
        goto    can_f
can_1   movwf   can_r1
        movlw   2
        movwf   can_mode
        movlw   0x51
        movwf   ADCON0
        goto    can_f
can_2   movwf   can_r2
        movlw   3
        movwf   can_mode
        movlw   0x59
        movwf   ADCON0
        goto    can_f
can_3   movwf   can_r3
        movlw   4
        movwf   can_mode
        movlw   0x61
        movwf   ADCON0
        goto    can_f
can_4   movwf   can_r4
        movlw   0
        movwf   can_mode
        movlw   0x41
        movwf   ADCON0
        goto    can_f

can_f

;       bsf     ADCON0,2  ;start conversion
;                         ;laisse le temps au condensateur d'acquisition
;                         ;de se charger sur le nouveau canal
;                         ;grosse interference inter canal avec 2 nop
;                         ;est fait dans maj_can2 

        return

maj_can2

        bsf     ADCON0,2

        return


;-----------------------------------------------------------------------------
;env 40us (10 + 30pour4pwm)

cal_pwm
        incf    pwm_cpt,f
        movf    pwm_cpt,w
        subwf   pwm_stp,w
        btfsc   STATUS,ZERO
        clrf    pwm_cpt
        btfsc   STATUS,ZERO
        clrf    pwm_cpt
	btfsc   STATUS,ZERO
	goto    det_i
        
        clrf    pwm_wrk

        movf    pwm_r0,w
        subwf   pwm_cpt,w ;cpt-cmd  f-w  0-2>C=1,2-0>C=0,0-0>C=0 cmd>cpt C=1
        clrw              ;ne modifie pas les flags
        btfss   STATUS,CARRY
        movlw   0x03
        iorwf   pwm_wrk,f

        movf    pwm_r1,w
        subwf   pwm_cpt,w   
        clrw              
        btfss   STATUS,CARRY
        movlw   0x0C
        iorwf   pwm_wrk,f

        movf    pwm_r2,w
        subwf   pwm_cpt,w   
        clrw              
        btfss   STATUS,CARRY
        movlw   0x30
        iorwf   pwm_wrk,f

        movf    pwm_r3,w
        subwf   pwm_cpt,w   
        clrw              
        btfss   STATUS,CARRY
        movlw   0xC0
        iorwf   pwm_wrk,f

        movf    pwm_wrk,w
        andwf   pwm_s03,w
        movwf   out_pwm1


        clrf    pwm_wrk

        movf    pwm_r4,w
        subwf   pwm_cpt,w ;cpt-cmd  f-w  0-2>C=1,2-0>C=0,0-0>C=0 cmd>cpt C=1
        clrw              ;ne modifie pas les flags
        btfss   STATUS,CARRY
        movlw   0x03
        iorwf   pwm_wrk,f

        movf    pwm_r5,w
        subwf   pwm_cpt,w   
        clrw              
        btfss   STATUS,CARRY
        movlw   0x0C
        iorwf   pwm_wrk,f

        movf    pwm_r6,w
        subwf   pwm_cpt,w   
        clrw              
        btfss   STATUS,CARRY
        movlw   0x30
        iorwf   pwm_wrk,f

        movf    pwm_r7,w
        subwf   pwm_cpt,w   
        clrw              
        btfss   STATUS,CARRY
        movlw   0xC0
        iorwf   pwm_wrk,f

        movf    pwm_wrk,w
        andwf   pwm_s47,w
        movwf   out_pwm2


        clrf    pwm_wrk

        movf    pwm_r8,w
        subwf   pwm_cpt,w ;cpt-cmd  f-w  0-2>C=1,2-0>C=0,0-0>C=0 cmd>cpt C=1
        clrw              ;ne modifie pas les flags
        btfss   STATUS,CARRY
        movlw   0x03
        iorwf   pwm_wrk,f

        movf    pwm_r9,w
        subwf   pwm_cpt,w   
        clrw              
        btfss   STATUS,CARRY
        movlw   0x0C
        iorwf   pwm_wrk,f

        movf    pwm_r10,w
        subwf   pwm_cpt,w   
        clrw              
        btfss   STATUS,CARRY
        movlw   0x30
        iorwf   pwm_wrk,f

        movf    pwm_r11,w
        subwf   pwm_cpt,w   
        clrw              
        btfss   STATUS,CARRY
        movlw   0xC0
        iorwf   pwm_wrk,f

        movf    pwm_wrk,w
        andwf   pwm_s8,w
        movwf   out_pwm3


        clrf    pwm_wrk

        movf    pwm_r12,w
        subwf   pwm_cpt,w ;cpt-cmd  f-w  0-2>C=1,2-0>C=0,0-0>C=0 cmd>cpt C=1
        clrw              ;ne modifie pas les flags
        btfss   STATUS,CARRY
        movlw   0x03
        iorwf   pwm_wrk,f

        movf    pwm_r13,w
        subwf   pwm_cpt,w   
        clrw              
        btfss   STATUS,CARRY
        movlw   0x0C
        iorwf   pwm_wrk,f

        movf    pwm_r14,w
        subwf   pwm_cpt,w   
        clrw              
        btfss   STATUS,CARRY
        movlw   0x30
        iorwf   pwm_wrk,f

        movf    pwm_r15,w
        subwf   pwm_cpt,w   
        clrw              
        btfss   STATUS,CARRY
        movlw   0xC0
        iorwf   pwm_wrk,f

        movf    pwm_wrk,w
        andwf   pwm_s12,w
        movwf   out_pwm4

        return


;-----------------------------------------------------------------------------

det_i
	bcf     cpt_i,7
	clrf	out_pwm1
	clrf	out_pwm2
	clrf	out_pwm3
	clrf	out_pwm4
        incf    cpt_i,f
	btfsc   cpt_i,4
	clrf    cpt_i
	bsf     cpt_i,7

        btfss   cpt_i,3
        goto    cp0xxx
        btfss   cpt_i,2
        goto    cp10xx
cp11xx  btfss   cpt_i,1
        goto    cp110x
cp111x  btfss   cpt_i,0
        goto    cp_14
        goto    cp_15
cp110x  btfss   cpt_i,0
        goto    cp_12
        goto    cp_13
cp10xx  btfss   cpt_i,1
        goto    cp100x
cp101x  btfss   cpt_i,0
        goto    cp_10
        goto    cp_11
cp100x  btfss   cpt_i,0    
        goto    cp_8
        goto    cp_9
cp0xxx  btfss   cpt_i,2
        goto    cp00xx
cp01xx  btfss   cpt_i,1
        goto    cp010x
cp011x  btfss   cpt_i,0
        goto    cp_6
        goto    cp_7
cp010x  btfss   cpt_i,0
        goto    cp_4
        goto    cp_5
cp00xx  btfss   cpt_i,1
        goto    cp000x
cp001x  btfss   cpt_i,0
        goto    cp_2
        goto    cp_3
cp000x  btfss   cpt_i,0    
        goto    cp_0
        goto    cp_1

cp_0    movf    pwm_s03,w
        andlw   0x03
	btfsc   STATUS,ZERO       
        movlw   0x01
        movwf   out_pwm1
        goto    cp_f

cp_1    movf    pwm_s03,w
        andlw   0x0c
	btfsc   STATUS,ZERO       
        movlw   0x04
        movwf   out_pwm1
        goto    cp_f

cp_2    movf    pwm_s03,w
        andlw   0x30
	btfsc   STATUS,ZERO       
        movlw   0x10
        movwf   out_pwm1
        goto    cp_f

cp_3    movf    pwm_s03,w
        andlw   0xc0
	btfsc   STATUS,ZERO       
        movlw   0x40
        movwf   out_pwm1
        goto    cp_f

cp_4    movf    pwm_s47,w
        andlw   0x03
	btfsc   STATUS,ZERO       
        movlw   0x01
        movwf   out_pwm2
        goto    cp_f

cp_5    movf    pwm_s47,w
        andlw   0x0c
	btfsc   STATUS,ZERO       
        movlw   0x04
        movwf   out_pwm2
        goto    cp_f

cp_6    movf    pwm_s47,w
        andlw   0x30
	btfsc   STATUS,ZERO       
        movlw   0x10
        movwf   out_pwm2
        goto    cp_f

cp_7    movf    pwm_s47,w
        andlw   0xc0
	btfsc   STATUS,ZERO       
        movlw   0x40
        movwf   out_pwm2
        goto    cp_f

cp_8    movf    pwm_s8,w
        andlw   0x03
	btfsc   STATUS,ZERO       
        movlw   0x01
        movwf   out_pwm3
        goto    cp_f

cp_9    movf    pwm_s8,w
        andlw   0x0c
	btfsc   STATUS,ZERO       
        movlw   0x04
        movwf   out_pwm3
        goto    cp_f

cp_10   movf    pwm_s8,w
        andlw   0x30
	btfsc   STATUS,ZERO       
        movlw   0x10
        movwf   out_pwm3
        goto    cp_f

cp_11   movf    pwm_s8,w
        andlw   0xc0
	btfsc   STATUS,ZERO       
        movlw   0x40
        movwf   out_pwm3
        goto    cp_f

cp_12   movf    pwm_s12,w
        andlw   0x03
	btfsc   STATUS,ZERO       
        movlw   0x01
        movwf   out_pwm4
        goto    cp_f

cp_13   movf    pwm_s12,w
        andlw   0x0c
	btfsc   STATUS,ZERO       
        movlw   0x04
        movwf   out_pwm4
        goto    cp_f

cp_14   movf    pwm_s12,w
        andlw   0x30
	btfsc   STATUS,ZERO       
        movlw   0x10
        movwf   out_pwm4
        goto    cp_f

cp_15   movf    pwm_s12,w
        andlw   0xc0
	btfsc   STATUS,ZERO       
        movlw   0x40
        movwf   out_pwm4
        goto    cp_f

cp_f    return


;-----------------------------------------------------------------------------

get_i
	bcf	cpt_i,7
	rrf	det_i2t,f
	rrf	det_i1t,f
	bcf	det_i2t,7
	btfss   PORTB,3
	bsf     det_i2t,7
	movlw   0x0f
	subwf	cpt_i,w
	btfss	STATUS,ZERO
	return
	movf    det_i1t,w
	movwf   det_i1
        iorwf   det_i1m,f
	movf    det_i2t,w
	movwf   det_i2
        iorwf   det_i2m,f
	return


;-----------------------------------------------------------------------------

cal_led
        clrw
        btfsc   cpt2,3
        movlw   0xff
        movwf   led_cli
        
        incf    led_cpt,f
        btfsc   led_cpt,1
        goto    led_1X
led_0X  btfss   led_cpt,0
        goto    led_0
        goto    led_1
led_1X  btfss   led_cpt,0
        goto    led_2
        goto    led_3


led_0
        movf    led_cli,w
        xorwf   led_in0,w
        iorwf   led_cl0,w
        andwf   led_on0,w
        movwf   out_out1
        movf    led_cli,w
        xorwf   led_in01,w
        iorwf   led_cl01,w
        andwf   led_on01,w
        andlw   0x0f
        movwf   out_out2
        bsf     out_out2,4
        goto    led_f

led_1
        movf    led_cli,w
        xorwf   led_in1,w
        iorwf   led_cl1,w
        andwf   led_on1,w
        movwf   out_out1
        movf    led_cli,w
        xorwf   led_in01,w
        iorwf   led_cl01,w
        andwf   led_on01,w
        movwf   out_out2
        swapf   out_out2,w
        andlw   0x0f
        movwf   out_out2
        bsf     out_out2,5
        goto    led_f

led_2
        movf    led_cli,w
        xorwf   led_in2,w
        iorwf   led_cl2,w
        andwf   led_on2,w
        movwf   out_out1
        movf    led_cli,w
        xorwf   led_in23,w
        iorwf   led_cl23,w
        andwf   led_on23,w
        andlw   0x0f
        movwf   out_out2
        bsf     out_out2,6
        goto    led_f

led_3
        movf    led_cli,w
        xorwf   led_in3,w
        iorwf   led_cl3,w
        andwf   led_on3,w
        movwf   out_out1
        movf    led_cli,w
        xorwf   led_in23,w
        iorwf   led_cl23,w
        andwf   led_on23,w
        movwf   out_out2
        swapf   out_out2,w
        andlw   0x0f
        movwf   out_out2
        bsf     out_out2,7
        goto    led_f

led_f
        return

;-----------------------------------------------------------------------------

maj_mat
        ;*** out_mat ***

        btfss out_mat,7
        bcf   PORTC,DATMAT
        bcf   PORTC,CLKMAT
        btfsc out_mat,7
        bsf   PORTC,DATMAT
        nop
        bsf   PORTC,CLKMAT

        btfss out_mat,6
        bcf   PORTC,DATMAT
        bcf   PORTC,CLKMAT
        btfsc out_mat,6
        bsf   PORTC,DATMAT
        nop
        bsf   PORTC,CLKMAT

        btfss out_mat,5
        bcf   PORTC,DATMAT
        bcf   PORTC,CLKMAT
        btfsc out_mat,5
        bsf   PORTC,DATMAT
        nop
        bsf   PORTC,CLKMAT

        btfss out_mat,4
        bcf   PORTC,DATMAT
        bcf   PORTC,CLKMAT
        btfsc out_mat,4
        bsf   PORTC,DATMAT
        nop
        bsf   PORTC,CLKMAT

        btfss out_mat,3
        bcf   PORTC,DATMAT
        bcf   PORTC,CLKMAT
        btfsc out_mat,3
        bsf   PORTC,DATMAT
        nop
        bsf   PORTC,CLKMAT

        btfss out_mat,2
        bcf   PORTC,DATMAT
        bcf   PORTC,CLKMAT
        btfsc out_mat,2
        bsf   PORTC,DATMAT
        nop
        bsf   PORTC,CLKMAT

        btfss out_mat,1
        bcf   PORTC,DATMAT
        bcf   PORTC,CLKMAT
        btfsc out_mat,1
        bsf   PORTC,DATMAT
        nop
        bsf   PORTC,CLKMAT

        btfss out_mat,0
        bcf   PORTC,DATMAT
        bcf   PORTC,CLKMAT
        btfsc out_mat,0
        bsf   PORTC,DATMAT
        nop
        bsf   PORTC,CLKMAT

        return

;-----------------------------------------------------------------------------

maj_out5
        ;*** out_out ***
        ;5

        btfss out_out5,7
        bcf   PORTB,DATOUT
        bcf   PORTB,CLKOUT
        btfsc out_out5,7
        bsf   PORTB,DATOUT
        nop
        bsf   PORTB,CLKOUT

        btfss out_out5,6
        bcf   PORTB,DATOUT
        bcf   PORTB,CLKOUT
        btfsc out_out5,6
        bsf   PORTB,DATOUT
        nop
        bsf   PORTB,CLKOUT

        btfss out_out5,5
        bcf   PORTB,DATOUT
        bcf   PORTB,CLKOUT
        btfsc out_out5,5
        bsf   PORTB,DATOUT
        nop
        bsf   PORTB,CLKOUT

        btfss out_out5,4
        bcf   PORTB,DATOUT
        bcf   PORTB,CLKOUT
        btfsc out_out5,4
        bsf   PORTB,DATOUT
        nop
        bsf   PORTB,CLKOUT

        btfss out_out5,3
        bcf   PORTB,DATOUT
        bcf   PORTB,CLKOUT
        btfsc out_out5,3
        bsf   PORTB,DATOUT
        nop
        bsf   PORTB,CLKOUT

        btfss out_out5,2
        bcf   PORTB,DATOUT
        bcf   PORTB,CLKOUT
        btfsc out_out5,2
        bsf   PORTB,DATOUT
        nop
        bsf   PORTB,CLKOUT

        btfss out_out5,1
        bcf   PORTB,DATOUT
        bcf   PORTB,CLKOUT
        btfsc out_out5,1
        bsf   PORTB,DATOUT
        nop
        bsf   PORTB,CLKOUT

        btfss out_out5,0
        bcf   PORTB,DATOUT
        bcf   PORTB,CLKOUT
        btfsc out_out5,0
        bsf   PORTB,DATOUT
        nop
        bsf   PORTB,CLKOUT

        return

maj_out4
        ;4

        btfss out_out4,7
        bcf   PORTB,DATOUT
        bcf   PORTB,CLKOUT
        btfsc out_out4,7
        bsf   PORTB,DATOUT
        nop
        bsf   PORTB,CLKOUT

        btfss out_out4,6
        bcf   PORTB,DATOUT
        bcf   PORTB,CLKOUT
        btfsc out_out4,6
        bsf   PORTB,DATOUT
        nop
        bsf   PORTB,CLKOUT

        btfss out_out4,5
        bcf   PORTB,DATOUT
        bcf   PORTB,CLKOUT
        btfsc out_out4,5
        bsf   PORTB,DATOUT
        nop
        bsf   PORTB,CLKOUT

        btfss out_out4,4
        bcf   PORTB,DATOUT
        bcf   PORTB,CLKOUT
        btfsc out_out4,4
        bsf   PORTB,DATOUT
        nop
        bsf   PORTB,CLKOUT

        btfss out_out4,3
        bcf   PORTB,DATOUT
        bcf   PORTB,CLKOUT
        btfsc out_out4,3
        bsf   PORTB,DATOUT
        nop
        bsf   PORTB,CLKOUT

        btfss out_out4,2
        bcf   PORTB,DATOUT
        bcf   PORTB,CLKOUT
        btfsc out_out4,2
        bsf   PORTB,DATOUT
        nop
        bsf   PORTB,CLKOUT

        btfss out_out4,1
        bcf   PORTB,DATOUT
        bcf   PORTB,CLKOUT
        btfsc out_out4,1
        bsf   PORTB,DATOUT
        nop
        bsf   PORTB,CLKOUT

        btfss out_out4,0
        bcf   PORTB,DATOUT
        bcf   PORTB,CLKOUT
        btfsc out_out4,0
        bsf   PORTB,DATOUT
        nop
        bsf   PORTB,CLKOUT

        return

maj_out3
        ;3

        btfss out_out3,7
        bcf   PORTB,DATOUT
        bcf   PORTB,CLKOUT
        btfsc out_out3,7
        bsf   PORTB,DATOUT
        nop
        bsf   PORTB,CLKOUT

        btfss out_out3,6
        bcf   PORTB,DATOUT
        bcf   PORTB,CLKOUT
        btfsc out_out3,6
        bsf   PORTB,DATOUT
        nop
        bsf   PORTB,CLKOUT

        btfss out_out3,5
        bcf   PORTB,DATOUT
        bcf   PORTB,CLKOUT
        btfsc out_out3,5
        bsf   PORTB,DATOUT
        nop
        bsf   PORTB,CLKOUT

        btfss out_out3,4
        bcf   PORTB,DATOUT
        bcf   PORTB,CLKOUT
        btfsc out_out3,4
        bsf   PORTB,DATOUT
        nop
        bsf   PORTB,CLKOUT

        btfss out_out3,3
        bcf   PORTB,DATOUT
        bcf   PORTB,CLKOUT
        btfsc out_out3,3
        bsf   PORTB,DATOUT
        nop
        bsf   PORTB,CLKOUT

        btfss out_out3,2
        bcf   PORTB,DATOUT
        bcf   PORTB,CLKOUT
        btfsc out_out3,2
        bsf   PORTB,DATOUT
        nop
        bsf   PORTB,CLKOUT

        btfss out_out3,1
        bcf   PORTB,DATOUT
        bcf   PORTB,CLKOUT
        btfsc out_out3,1
        bsf   PORTB,DATOUT
        nop
        bsf   PORTB,CLKOUT

        btfss out_out3,0
        bcf   PORTB,DATOUT
        bcf   PORTB,CLKOUT
        btfsc out_out3,0
        bsf   PORTB,DATOUT
        nop
        bsf   PORTB,CLKOUT

        return

maj_out2
        ;2

        btfss out_out2,7
        bcf   PORTB,DATOUT
        bcf   PORTB,CLKOUT
        btfsc out_out2,7
        bsf   PORTB,DATOUT
        nop
        bsf   PORTB,CLKOUT

        btfss out_out2,6
        bcf   PORTB,DATOUT
        bcf   PORTB,CLKOUT
        btfsc out_out2,6
        bsf   PORTB,DATOUT
        nop
        bsf   PORTB,CLKOUT

        btfss out_out2,5
        bcf   PORTB,DATOUT
        bcf   PORTB,CLKOUT
        btfsc out_out2,5
        bsf   PORTB,DATOUT
        nop
        bsf   PORTB,CLKOUT

        btfss out_out2,4
        bcf   PORTB,DATOUT
        bcf   PORTB,CLKOUT
        btfsc out_out2,4
        bsf   PORTB,DATOUT
        nop
        bsf   PORTB,CLKOUT

        btfss out_out2,3
        bcf   PORTB,DATOUT
        bcf   PORTB,CLKOUT
        btfsc out_out2,3
        bsf   PORTB,DATOUT
        nop
        bsf   PORTB,CLKOUT

        btfss out_out2,2
        bcf   PORTB,DATOUT
        bcf   PORTB,CLKOUT
        btfsc out_out2,2
        bsf   PORTB,DATOUT
        nop
        bsf   PORTB,CLKOUT

        btfss out_out2,1
        bcf   PORTB,DATOUT
        bcf   PORTB,CLKOUT
        btfsc out_out2,1
        bsf   PORTB,DATOUT
        nop
        bsf   PORTB,CLKOUT

        btfss out_out2,0
        bcf   PORTB,DATOUT
        bcf   PORTB,CLKOUT
        btfsc out_out2,0
        bsf   PORTB,DATOUT
        nop
        bsf   PORTB,CLKOUT

        return

maj_out1
        ;1

        btfss out_out1,7
        bcf   PORTB,DATOUT
        bcf   PORTB,CLKOUT
        btfsc out_out1,7
        bsf   PORTB,DATOUT
        nop
        bsf   PORTB,CLKOUT

        btfss out_out1,6
        bcf   PORTB,DATOUT
        bcf   PORTB,CLKOUT
        btfsc out_out1,6
        bsf   PORTB,DATOUT
        nop
        bsf   PORTB,CLKOUT

        btfss out_out1,5
        bcf   PORTB,DATOUT
        bcf   PORTB,CLKOUT
        btfsc out_out1,5
        bsf   PORTB,DATOUT
        nop
        bsf   PORTB,CLKOUT

        btfss out_out1,4
        bcf   PORTB,DATOUT
        bcf   PORTB,CLKOUT
        btfsc out_out1,4
        bsf   PORTB,DATOUT
        nop
        bsf   PORTB,CLKOUT

        btfss out_out1,3
        bcf   PORTB,DATOUT
        bcf   PORTB,CLKOUT
        btfsc out_out1,3
        bsf   PORTB,DATOUT
        nop
        bsf   PORTB,CLKOUT

        btfss out_out1,2
        bcf   PORTB,DATOUT
        bcf   PORTB,CLKOUT
        btfsc out_out1,2
        bsf   PORTB,DATOUT
        nop
        bsf   PORTB,CLKOUT

        btfss out_out1,1
        bcf   PORTB,DATOUT
        bcf   PORTB,CLKOUT
        btfsc out_out1,1
        bsf   PORTB,DATOUT
        nop
        bsf   PORTB,CLKOUT

        btfss out_out1,0
        bcf   PORTB,DATOUT
        bcf   PORTB,CLKOUT
        btfsc out_out1,0
        bsf   PORTB,DATOUT
        nop
        bsf   PORTB,CLKOUT

        ;L

        bsf   PORTB,LODOUT
        nop
        nop
        nop
        bcf   PORTB,LODOUT

        return

;-----------------------------------------------------------------------------

maj_pwm
        ;*** out_pwm ***
        ;4

        btfss out_pwm4,7
        bcf   PORTB,DATPWM
        bcf   PORTB,CLKPWM
        btfsc out_pwm4,7
        bsf   PORTB,DATPWM
        nop
        bsf   PORTB,CLKPWM

        btfss out_pwm4,6
        bcf   PORTB,DATPWM
        bcf   PORTB,CLKPWM
        btfsc out_pwm4,6
        bsf   PORTB,DATPWM
        nop
        bsf   PORTB,CLKPWM

        btfss out_pwm4,5
        bcf   PORTB,DATPWM
        bcf   PORTB,CLKPWM
        btfsc out_pwm4,5
        bsf   PORTB,DATPWM
        nop
        bsf   PORTB,CLKPWM

        btfss out_pwm4,4
        bcf   PORTB,DATPWM
        bcf   PORTB,CLKPWM
        btfsc out_pwm4,4
        bsf   PORTB,DATPWM
        nop
        bsf   PORTB,CLKPWM

        btfss out_pwm4,3
        bcf   PORTB,DATPWM
        bcf   PORTB,CLKPWM
        btfsc out_pwm4,3
        bsf   PORTB,DATPWM
        nop
        bsf   PORTB,CLKPWM

        btfss out_pwm4,2
        bcf   PORTB,DATPWM
        bcf   PORTB,CLKPWM
        btfsc out_pwm4,2
        bsf   PORTB,DATPWM
        nop
        bsf   PORTB,CLKPWM

        btfss out_pwm4,1
        bcf   PORTB,DATPWM
        bcf   PORTB,CLKPWM
        btfsc out_pwm4,1
        bsf   PORTB,DATPWM
        nop
        bsf   PORTB,CLKPWM

        btfss out_pwm4,0
        bcf   PORTB,DATPWM
        bcf   PORTB,CLKPWM
        btfsc out_pwm4,0
        bsf   PORTB,DATPWM
        nop
        bsf   PORTB,CLKPWM

        ;3

        btfss out_pwm3,7
        bcf   PORTB,DATPWM
        bcf   PORTB,CLKPWM
        btfsc out_pwm3,7
        bsf   PORTB,DATPWM
        nop
        bsf   PORTB,CLKPWM

        btfss out_pwm3,6
        bcf   PORTB,DATPWM
        bcf   PORTB,CLKPWM
        btfsc out_pwm3,6
        bsf   PORTB,DATPWM
        nop
        bsf   PORTB,CLKPWM

        btfss out_pwm3,5
        bcf   PORTB,DATPWM
        bcf   PORTB,CLKPWM
        btfsc out_pwm3,5
        bsf   PORTB,DATPWM
        nop
        bsf   PORTB,CLKPWM

        btfss out_pwm3,4
        bcf   PORTB,DATPWM
        bcf   PORTB,CLKPWM
        btfsc out_pwm3,4
        bsf   PORTB,DATPWM
        nop
        bsf   PORTB,CLKPWM

        btfss out_pwm3,3
        bcf   PORTB,DATPWM
        bcf   PORTB,CLKPWM
        btfsc out_pwm3,3
        bsf   PORTB,DATPWM
        nop
        bsf   PORTB,CLKPWM

        btfss out_pwm3,2
        bcf   PORTB,DATPWM
        bcf   PORTB,CLKPWM
        btfsc out_pwm3,2
        bsf   PORTB,DATPWM
        nop
        bsf   PORTB,CLKPWM

        btfss out_pwm3,1
        bcf   PORTB,DATPWM
        bcf   PORTB,CLKPWM
        btfsc out_pwm3,1
        bsf   PORTB,DATPWM
        nop
        bsf   PORTB,CLKPWM

        btfss out_pwm3,0
        bcf   PORTB,DATPWM
        bcf   PORTB,CLKPWM
        btfsc out_pwm3,0
        bsf   PORTB,DATPWM
        nop
        bsf   PORTB,CLKPWM

        ;2

        btfss out_pwm2,7
        bcf   PORTB,DATPWM
        bcf   PORTB,CLKPWM
        btfsc out_pwm2,7
        bsf   PORTB,DATPWM
        nop
        bsf   PORTB,CLKPWM

        btfss out_pwm2,6
        bcf   PORTB,DATPWM
        bcf   PORTB,CLKPWM
        btfsc out_pwm2,6
        bsf   PORTB,DATPWM
        nop
        bsf   PORTB,CLKPWM

        btfss out_pwm2,5
        bcf   PORTB,DATPWM
        bcf   PORTB,CLKPWM
        btfsc out_pwm2,5
        bsf   PORTB,DATPWM
        nop
        bsf   PORTB,CLKPWM

        btfss out_pwm2,4
        bcf   PORTB,DATPWM
        bcf   PORTB,CLKPWM
        btfsc out_pwm2,4
        bsf   PORTB,DATPWM
        nop
        bsf   PORTB,CLKPWM

        btfss out_pwm2,3
        bcf   PORTB,DATPWM
        bcf   PORTB,CLKPWM
        btfsc out_pwm2,3
        bsf   PORTB,DATPWM
        nop
        bsf   PORTB,CLKPWM

        btfss out_pwm2,2
        bcf   PORTB,DATPWM
        bcf   PORTB,CLKPWM
        btfsc out_pwm2,2
        bsf   PORTB,DATPWM
        nop
        bsf   PORTB,CLKPWM

        btfss out_pwm2,1
        bcf   PORTB,DATPWM
        bcf   PORTB,CLKPWM
        btfsc out_pwm2,1
        bsf   PORTB,DATPWM
        nop
        bsf   PORTB,CLKPWM

        btfss out_pwm2,0
        bcf   PORTB,DATPWM
        bcf   PORTB,CLKPWM
        btfsc out_pwm2,0
        bsf   PORTB,DATPWM
        nop
        bsf   PORTB,CLKPWM

        ;1

        btfss out_pwm1,7
        bcf   PORTB,DATPWM
        bcf   PORTB,CLKPWM
        btfsc out_pwm1,7
        bsf   PORTB,DATPWM
        nop
        bsf   PORTB,CLKPWM

        btfss out_pwm1,6
        bcf   PORTB,DATPWM
        bcf   PORTB,CLKPWM
        btfsc out_pwm1,6
        bsf   PORTB,DATPWM
        nop
        bsf   PORTB,CLKPWM

        btfss out_pwm1,5
        bcf   PORTB,DATPWM
        bcf   PORTB,CLKPWM
        btfsc out_pwm1,5
        bsf   PORTB,DATPWM
        nop
        bsf   PORTB,CLKPWM

        btfss out_pwm1,4
        bcf   PORTB,DATPWM
        bcf   PORTB,CLKPWM
        btfsc out_pwm1,4
        bsf   PORTB,DATPWM
        nop
        bsf   PORTB,CLKPWM

        btfss out_pwm1,3
        bcf   PORTB,DATPWM
        bcf   PORTB,CLKPWM
        btfsc out_pwm1,3
        bsf   PORTB,DATPWM
        nop
        bsf   PORTB,CLKPWM

        btfss out_pwm1,2
        bcf   PORTB,DATPWM
        bcf   PORTB,CLKPWM
        btfsc out_pwm1,2
        bsf   PORTB,DATPWM
        nop
        bsf   PORTB,CLKPWM

        btfss out_pwm1,1
        bcf   PORTB,DATPWM
        bcf   PORTB,CLKPWM
        btfsc out_pwm1,1
        bsf   PORTB,DATPWM
        nop
        bsf   PORTB,CLKPWM

        btfss out_pwm1,0
        bcf   PORTB,DATPWM
        bcf   PORTB,CLKPWM
        btfsc out_pwm1,0
        bsf   PORTB,DATPWM
        nop
        bsf   PORTB,CLKPWM

        ;L

        bsf   PORTB,LODPWM
        nop
        nop
        nop
        bcf   PORTB,LODPWM

        return

;-----------------------------------------------------------------------------

maj_in
        bcf   PORTC,DATMAT
        bcf   PORTC,CLKMAT
        nop
        nop

        bsf   PORTC,CLKMAT
        movlw 0x0f
        andwf mat_in0,f
        comf  PORTC,w
        bcf   PORTC,CLKMAT
        bsf   PORTC,DATMAT
        andlw 0x0f
        iorwf mat_in0,f
        movwf mat_tmp
        swapf mat_tmp,w
        iorwf mat_in0,f

        bsf   PORTC,CLKMAT
        movlw 0x0f
        andwf mat_in1,f
        comf  PORTC,w
        bcf   PORTC,CLKMAT
        andlw 0x0f
        iorwf mat_in1,f
        movwf mat_tmp
        swapf mat_tmp,w
        iorwf mat_in1,f

        bsf   PORTC,CLKMAT
        movlw 0x0f
        andwf mat_in2,f
        comf  PORTC,w
        bcf   PORTC,CLKMAT
        andlw 0x0f
        iorwf mat_in2,f
        movwf mat_tmp
        swapf mat_tmp,w
        iorwf mat_in2,f

        bsf   PORTC,CLKMAT
        movlw 0x0f
        andwf mat_in3,f
        comf  PORTC,w
        bcf   PORTC,CLKMAT
        andlw 0x0f
        iorwf mat_in3,f
        movwf mat_tmp
        swapf mat_tmp,w
        iorwf mat_in3,f

        bsf   PORTC,CLKMAT
        movlw 0x0f
        andwf mat_in4,f
        comf  PORTC,w
        bcf   PORTC,CLKMAT
        andlw 0x0f
        iorwf mat_in4,f
        movwf mat_tmp
        swapf mat_tmp,w
        iorwf mat_in4,f

        bsf   PORTC,CLKMAT
        movlw 0x0f
        andwf mat_in5,f
        comf  PORTC,w
        bcf   PORTC,CLKMAT
        andlw 0x0f
        iorwf mat_in5,f
        movwf mat_tmp
        swapf mat_tmp,w
        iorwf mat_in5,f

        bsf   PORTC,CLKMAT
        movlw 0x0f
        andwf mat_in6,f
        comf  PORTC,w
        bcf   PORTC,CLKMAT
        andlw 0x0f
        iorwf mat_in6,f
        movwf mat_tmp
        swapf mat_tmp,w
        iorwf mat_in6,f

        bsf   PORTC,CLKMAT
        movlw 0x0f
        andwf mat_in7,f
        comf  PORTC,w
        bcf   PORTC,CLKMAT
        andlw 0x0f
        iorwf mat_in7,f
        movwf mat_tmp
        swapf mat_tmp,w
        iorwf mat_in7,f

        bsf   PORTC,CLKMAT
        movlw 0x0f
        andwf mat_in8,f
        comf  PORTC,w
        bcf   PORTC,CLKMAT
        andlw 0x0f
        iorwf mat_in8,f
        movwf mat_tmp
        swapf mat_tmp,w
        iorwf mat_in8,f

        return

;-----------------------------------------------------------------------------

servo_h
        btfss   PIR1,CCP1IF
        goto    servo2_h
        btfsc   CCP1CON,0   ;continue 8 (output is high)
        goto    servo2_h

        bcf     T1CON,TMR1ON

        movf    ccpf_h,w
        movwf   CCPR1H
        movf    ccpf_l,w
        movwf   CCPR1L

        bsf     CCP1CON,0   ;set 9 (next match output low)
        bcf     PIR1,CCP1IF
        bsf     T1CON,TMR1ON

servo2_h 
        btfss   PIR2,CCP2IF
        return
        btfsc   CCP2CON,0   ;continue 8 (output is high)
        return

        bcf     T1CON,TMR1ON

        movf    ccp2f_h,w
        movwf   CCPR2H
        movf    ccp2f_l,w
        movwf   CCPR2L

        bsf     CCP2CON,0   ;set 9 (next match output low)
        bcf     PIR2,CCP2IF
        bsf     T1CON,TMR1ON

        return


;-----------------------------------------------------------------------------

servo_l
        btfss   PIR2,CCP2IF  ;at ccp2 fall then calculate ccp1 and ccp2
        return
        btfss   CCP2CON,0    ;continue 9 (output is low)
        return

        bcf     T1CON,TMR1ON 
        
        movf    ccpr_h,w     ;next rising edge
        addlw   20           ;period 20 ms
        movwf   ccpr_h
        movwf   CCPR1H
        clrf    CCPR1L

        movwf   ccpf_h       ;next falling edge
        ;movlw   0x7d         ;125*4=500us
        movlw   0x64         ;100*4=400us
        addwf   srv_0,w      ;400us+3*4us*valeur 400->3400us
        btfsc   STATUS,CARRY
        incf    ccpf_h,f
        addwf   srv_0,w
        btfsc   STATUS,CARRY
        incf    ccpf_h,f
        addwf   srv_0,w
        btfsc   STATUS,CARRY
        incf    ccpf_h,f
        movwf   ccpf_l
        bcf     CCP1CON,0   ;set 8 (next match output high)
        bcf     PIR1,CCP1IF

        movf    ccpr_h,w
        addlw   10           ;second servo pulse shifted 10ms
        movwf   CCPR2H
        clrf    CCPR2L

        movwf   ccp2f_h      ;next falling edge
        movlw   0x64         ;100*4=400us
        addwf   srv_1,w      ;400us+3*4us*valeur 400->3400us
        btfsc   STATUS,CARRY
        incf    ccp2f_h,f
        addwf   srv_1,w
        btfsc   STATUS,CARRY
        incf    ccp2f_h,f
        addwf   srv_1,w
        btfsc   STATUS,CARRY
        incf    ccp2f_h,f
        movwf   ccp2f_l
        bcf     CCP2CON,0   ;set 8 (next match output high)
        bcf     PIR2,CCP2IF

        bsf     T1CON,TMR1ON

        return

;-----------------------------------------------------------------------------

        END


