;-----------------------------------------------------------------------------
;             873RS34.ASM version <<<<V3.3>>>> du 15/02/2020 
;             INCOMPATIBLE avec la V2.0 ou V2.1 (matriel ou logiciel PC)
;
; UTS2000 - Site    : http://www.la-tour.info/uts/uts_index.html
;
;             40 ENTREES/SORTIES SUPPLEMENTAIRES et QUARTZ  20 Mhz en V3
;             Sorties PWM = Sens + Enable L293 !!! (Modification schma et CI)
;
;             Utilisation de la rs232 en moniteur sous IT
;             Mise en place du module can sous IT Timer0
;             Mise en place du module pwm sous IT Timer0
;             Dcalage pour les 3 groupes de sortie:
;             - out, - mat-out, - pwm
;              9 . Passage a 50 Hz
;             10 . Support du capteur de courant
;             11 . Amlioration de la liaison srie
;             12 . Rajout d'un servo (pour plaque tournante)
;             13 . Modification du moniteur (maintenant adressable (8 adresses))
;             14 . Version  8 Mhz commente JLF, libration de 10 registres
;                  Liaison srie  19200 Bd/s
;                  Modif lecture CAN + comparaison adresse de la carte
;             30 . Nouvelle implantation des sorties
;                  Version  20 Mhz + 40 entres + 40 sorties supplmentaires,
;                  Liaison srie  38400 Bd/s, passage  62 Hz pour PWM voies.
;             31 . Nouvelle implantation des sorties
;                  Nouveau circuit externe de dtection d'occupation des cantons.
;                  Utilisation 1 bit de sens + 1 bit Enable L293D pour PWM,
;                  Commande de 5 servos par dfaut sans perturbation.
;             33 . Lecture des entres multiplexes avec une tempo plus grande entre deux lectures de colonne.
;                  Amliore la lecture d'un clavier multiplex.
;             34 . Diverses variantes pour le signal vers les servo, autres ques 1,5 ms, 2msec et 2,5msec pour un dbattement de 180.
;                  5 Type de version suivnant le type de servo command. Voir SSPG_CPT7.
;                    873rs34-ADR0-1.002.00ms.HEX = Servo type 1 => Crneau =  1,00 msec  |  1,50 msec  |   2,00 msec
;                    873rs34-ADR0-0.882.11ms.HEX = Servo type 2 => Crneau =  0,88 msec  |  1,50 msec  |   2,11 msec
;                    873rs34-ADR0-0.782.21ms.HEX = Servo type 3 => Crneau =  0,78 msec  |  1,50 msec  |   2,21 msec
;                    873rs34-ADR0-0.682.31ms.HEX = Servo type 4 => Crneau =  0,68 msec  |  1,50 msec  |   2,31 msec
;                    873rs34-ADR0-0.572.41ms.HEX = Servo type 5 => Crneau =  0,57 msec  |  1,50 msec  |   2,41 msec
;
; Entres/Sorties en V3.0 = Circuit imprim V2.1 avec 2 coupures + 2 straps
; et cblage des trois nouveaux connecteurs d'interfaces.
; Entres/Sorties en V3.2 = Circuit imprim V2.1 + Modif V3.0 + recablage des L293D
;
; Utilisation des pattes du PIC - HHH = Horloge, VVV = Validation
;
;  PORTA
;    (0) Entre analogique potentiomtre (0)
;    (1) Entre analogique potentiomtre (1)
;    (2) Entre analogique potentiomtre (2)
;    (3) Entre analogique potentiomtre (3)
;    (4) Vers led jaune clignotante
;    (5) Entre analogique potentiomtre (4)
;  PORTB
;    (0) Entres numriques multiplexes, entre bit (2)
;    (1) HHH**Sortie horloge vers registre 74HCT4094 voies PWM
;        HHH--Sortie horloge vers registre 74HCT4094 entres multiplexes (ancien C(4))
;        HHH<<Sortie horloge vers registre 74HCT165 dtection d'occupation des cantons
;    (2) VVV**Sortie validation vers registre 74HCT4094 voies PWM
;    (3)    <<Entre donnes 74HCT165 dtection d'occupation des cantons
;    (4) VVV<<Sortie validation vers registre 74HCT165 dtection d'occupation des cantons
;    (5)    ++Sortie donnes vers registre 74HCT4094 sorties ULN2008 [1]
;    (6) VVV++Sortie validation vers registre 74HCT4094 sorties ULN2008 [1]
;        VVV//Sortie validation vers registre 74HCT4094 sorties ULN2008 [2]
;        VVV::Sortie validation vers registre CD4014 entres numriques directes [2]
;    (7) HHH++Sortie horloge vers registre 74HCT4094 sorties ULN2008 [1]
;        HHH//Sortie horloge vers registre 74HCT4094 sorties ULN2008 [2]
;        HHH::Sortie horloge vers registre 74HCT165 entres numriques directes [2]
;  PORTC
;    (0) Entres numriques multiplexes, entre bit (0)
;    (1) Entres numriques multiplexes, entre bit (1)
;    (2) Crneau de 1  2 msec vers les servos (CCP1)
;    (3) Entres numriques multiplexes, entre bit (3)
;    (4)    ::Entre donnes 74HCT165 entres numriques directes (2]
;    (5)    --Sortie donnes vers registre 74HCT4094 entres multiplexes
;           **Sortie donnes vers registre 74HCT4094 voies PWM (ancien B(4))
;           //Sortie donnes vers registre 74HCT4094[2] sorties ULN2008 [2]
;    (6) Sortie du PIC vers RS232
;    (7) Entre du PIC vers RS232
;
;
; Commandes depuis le PC :
;  0x00 re-synchronise la communication srie avec le PC (ne fait rien)
;  0x01 ping carte n1 et slection de la carte n1, retourne un octet =  0x83
;  0x11 ping carte n2 et slection de la carte n2, retourne un octet =  0x83
;  0x21 ping carte n3 et slection de la carte n3, retourne un octet =  0x83
;  0x31 ping carte n4 et slection de la carte n4, retourne un octet =  0x83
;  0x02 Met  jour le pointeur d'adresse 'FSR' avec l'octet suivant reu
;  0x03 Ecrit dans le registre point par [FSR] avec l'octet suivant reu
;  0x04 Lit dans le registre point par [FSR], le PIC renvoi un octet
;  0x05 Incrmente le pointeur 'FSR' et crit dans le registre point par [FSR] avec l'octet suivant reu
;  0x06 Incrmente le pointeur 'FSR' et lit dans le registre point par [FSR], le PIC renvoi un octet
;  0x07 Incrmente le pointeur 'FSR' et lit puis fface dans le registre point par [FSR], le PIC renvoi un octet
;  0x08 - 0X0F Transfre global (Ecriture + lecture)
;  0X10 - 0XFF Seul les 4 bits de poids faibles sont traits
;
; rx    0    1    2    3    4    5    6     7     8
; tx        0x83           data      +data +data
;
; rx2             adr  data      +data            data
; tx2                                             data
;
; Le ping permet galement de slectionner la carte suivant son adresse CARDADR (0  3)
; En fonction de la carte, il faut mettre la commande ping suivante  :
; Adresse =         0     1     2     3
; Octet   =      0x01  0x11  0x21  0x31
; Touches = ALT + 001 + 017 + 033 + 049 (sous simulateur ISIS)
;
; Avec un quart  20 Mhz => Une instruction = 0,2 sec
;
; A 19200 Bd, temps de transmission d'un caractre = 520 s
; A 38400 Bd, temps de transmission d'un caractre = 260 s doit tre > TMR0 (250s) !
;
; Transfert depuis le PC = 1 octet slection de carte + 1 octet de cmd + 47 octets vers le PIC
;                        = 1 octet de rponse + 23 in = 71 octets
;                        = Le PIC commence  rpondre  partir du deuxime octet de donne reu
;                        = et envoi un octet tous les 2 octets reu.
;                           38400 Bd, l'change dure = 13ms (en V3.2)
;
;
	LIST		P=PIC16F873		; Dfinition du processeur
	#include	<p16F873.inc>	; Dfinitions des variables

;	__CONFIG   _CP_OFF & _DEBUG_OFF & _WRT_OFF & _CPD_OFF & _LVP_OFF & _BODEN_OFF & _PWRTE_ON & _WDT_OFF & _HS_OSC
	__CONFIG   _CP_OFF & _DEBUG_OFF & _WRT_ENABLE_OFF & _CPD_OFF & _LVP_OFF & _BODEN_OFF & _PWRTE_ON & _WDT_OFF & _HS_OSC
;	P16F873 =  "_WRT_ENABLE_OFF",   P16F873A = "_WRT_OFF"
;
	ERRORLEVEL  -302

type_servo equ 1	; Compilation conditionnel de code suivant le type de servo.
;                     5 Type de version suivnant le type de servo command. Voir SSPG_CPT7.
;                       1 - 873rs34-ADR0-1.002.00ms.HEX = Servo type 1 => Crneau =  1,00 msec  |  1,50 msec  |   2,00 msec
;                       2 - 873rs34-ADR0-0.882.11ms.HEX = Servo type 2 => Crneau =  0,88 msec  |  1,50 msec  |   2,11 msec
;                       3 - 873rs34-ADR0-0.782.21ms.HEX = Servo type 3 => Crneau =  0,78 msec  |  1,50 msec  |   2,21 msec
;                       4 - 873rs34-ADR0-0.682.31ms.HEX = Servo type 4 => Crneau =  0,68 msec  |  1,50 msec  |   2,31 msec
;                       5 - 873rs34-ADR0-0.572.41ms.HEX = Servo type 5 => Crneau =  0,57 msec  |  1,50 msec  |   2,41 msec

CARDADR EQU 0		; Adresse de la carte mre (0-7) = Adresse sur la liaison RS232
					; variable  positionner pour produire 873RS31_1.HEX, 873RS31_2.HEX...

;-----------------------------------------------------------------------------
; DEFINITIONS et MACROS
; BANK2 et BANK 3 pas utilises, sinon modifier les macros BANK0 et BANK1

PAS_VITESSE	EQU	.64	; Nombre de pas disponibles pour la vitesse sur un canton
;				  Valeur de 'pwm_rX' = 0  'PAS_VITESSE'

CLKOUT  EQU     7	; de PORTB horloge    (1)et(2)  74HCT4094 vers relais et leds
LODOUT  EQU     6	; de PORTB validation (1)et(2)  74HCT4094 vers relais et leds
DATOUT  EQU     5	; de PORTB donnes    (1)       74HCT4094 vers relais et leds
DATOUT2 EQU     5	; de PORTC donnes    (2)       74HCT4094 vers relais et leds
DATIN2  EQU     4	; de PORTC donnes in (2)       75HCT165 depuis entres directes

CLKPWM  EQU     1	; de PORTB horloge    74HCT4094 vers les voies
LODPWM  EQU     2	; de PORTB validation 74HCT4094 vers les voies
DATPWM  EQU     5	; de PORTC donnes    74HCT4094 vers les voies En V3.2 B(4) vers C(5)

DATDEC2 EQU     3	; de PORTB donnes    dtection des cantons 75HCT165 depuis entres directes
LODDEC2 EQU     4	; de PORTB de chargement des donnes

CLKMAT  EQU     1	; de PORTB horloge    74HCT4094 vers matrice des entres. En V3.2 C(4) vers B(1)

DATMAT  EQU     5	; de PORTC donnes    74HCT4094 vers matrice des entres
rs_sel  EQU     0	; indicateurs.rs_sel = bit de slection de la carte
rs_env  EQU     1	; indicateurs.rs_env = bit d'envoi d'un caractre une fois sur deux

BANK0	macro				; passer en banque0
	bcf	STATUS,RP0
	endm

BANK1	macro				; passer en banque1
	bsf	STATUS,RP0
	endm

CK_DOWN_PWM macro
	bcf   PORTC,DATPWM
	nop
	bcf   PORTB,CLKPWM
	nop
	endm

CK_UP_PWM macro
	bsf   PORTC,DATPWM
	nop
	bsf   PORTB,CLKPWM
	nop
	endm

CK_DOWN_OUT macro
	bcf   PORTC,DATOUT2
	nop
	bcf   PORTB,CLKOUT
	endm

CK_UP_OUT macro
	bsf   PORTC,DATOUT2
	nop
	bsf   PORTB,CLKOUT
	nop
	endm

CK_UD_DAT macro
	bsf   PORTB,CLKMAT	; En V3.0 pour retrouver des sorties  1
	nop
	bcf   PORTB,CLKMAT	; Avant de commencer.
	nop
	endm

;-----------------------------------------------------------------------------
; ADRESSES  DES  VARIABLES
;
; Registres d'change avec le PC (ECRITURE depuis le PC)
; Registres de variables pour criture depuis le PC
;
; Donnes en BANK0 de 0x20  0x7F (96 adresses)
;--------VARIABLES EN ECRITURES -----------
	CBLOCK	0x20	;    Dbut de la zone variables
pwm_r0	: 1	; Valeur de la vitesse sur le canton 0 (= 0  63)
pwm_r1	: 1	;     //  1
pwm_r2	: 1	;     //  2
pwm_r3	: 1	;     //  3
pwm_r4	: 1	;
pwm_r5	: 1	;
pwm_r6	: 1	;
pwm_r7	: 1	;
pwm_r8	: 1	;
pwm_r9	: 1	;
pwm_r10	: 1	;
pwm_r11	: 1	;
pwm_r12	: 1	;
pwm_r13	: 1	;
pwm_r14	: 1	;     // 14
pwm_r15	: 1	;     // 15
pwm_s0_7	: 1	; Valeur sens de dplacement 1 bit /canton 0  7
pwm_s8_15	: 1	; Valeur sens de dplacement 1 bit /canton 8  15
out_out3	: 1	; Valeur des bits des sorties   1   7  (IC14-ULN2008 [1])
out_out4	: 1	;   //                          8  15  (IC15-ULN2008 [1])
out_out5	: 1	;   //                         16  23  (IC16-ULN2008 [1])
out_out21	: 1	; Valeur des bits des sorties           (ULN2008 [2]) V3.0
out_out22	: 1	;   //
out_out23	: 1	;   //
out_out24	: 1	;   //
out_out25	: 1	;   //
led_on0	: 1	; Leds 00-07 allumes             (1bit = une led)
led_on1	: 1	; Leds 12-19 allumes             ( bit = 1 = on )
led_on01	: 1	; Leds 08-11 et 20-23 allumes
led_on2	: 1	; Leds 24-31 allumes
led_on3	: 1	; Leds 36-43 allumes
led_on23	: 1	; Leds 32-35 et 43-47 allumes
led_cl0	: 1	; Leds 00-07 clignotantes         (1bit = une led)
led_cl1	: 1	; Leds 12-19 clignotantes         ( bit = 1 = cli)
led_cl01	: 1	; Leds 08-11 et 20-23 clignotantes
led_cl2	: 1	; Leds 24-31 clignotantes
led_cl3	: 1	; Leds 36-43 clignotantes
led_cl23	: 1	; Leds 32-35 et 43-47 clignotantes
led_in0	: 1	; Leds 00-07 clignotant invers    (1bit = une led)
led_in1	: 1	; Leds 12-19 clignotant invers    ( bit = 1 = inv)
led_in01	: 1	; Leds 08-11 et 20-23 clignotant invers
led_in2	: 1	; Leds 24-31 clignotant invers
led_in3	: 1	; Leds 36-43 clignotant invers
led_in23	: 1	; Leds 32-35 et 43-47 clignotant invers
srv_0		: 1	; Valeur de consigne du servo 0
srv_1		: 1	;     //                      1
srv_2		: 1	;     //                      2
srv_3		: 1	;     //                      3
srv_4		: 1	;     //                      4
ruf1		: 1   ; LIBRE rserv utilisation future
ruf2		: 1   ; LIBRE rserv utilisation future
ruf3		: 1   ; LIBRE rserv utilisation future
;
;
; ---------VARIABLES EN LECTURE -----------
can_r0	: 1	; Valeur lue sur l'entre analogique (0) ( = 0  255 )
can_r1	: 1	;   //                               (1)
can_r2	: 1	;   //                               (2)
can_r3	: 1	;   //                               (3)
can_r4	: 1	;   //                               (4)
in_in1 	: 1	; Valeurs des bits des 40 entres (75HCT165 [2]) V3.0
in_in2 	: 1	;   //
in_in3	: 1	;   //
in_in4	: 1	;   //
in_in5	: 1	;   //
det_i1	: 1	; Un bit par canton, = 1 si le canton est occup
det_i2	: 1	;   //
ruf11		: 1   ; LIBRE rserv utilisation future
;
;
; --------VARIABLES EN LECTURE ET EFFACEMENT -----------
det_i1m	: 1	; Un bit par canton, le passage  l'tat 1 est mmoris
det_i2m	: 1	; Tous ces Octets sont effacs par la lecture de ces registres.
mat_in0	: 1	; 4bits d'entres colonne INA + ligne IN3  IN0 + 4 bits mmorisation des
mat_in1	: 1	;   //  INB + ligne IN3  IN0                       tats, effacs par
mat_in2	: 1	;   //  INC + ligne IN3  IN0                       la lecture de ces
mat_in3	: 1	;   //  IND + ligne IN3  IN0                       registres.
mat_in4	: 1	;   //  INE + ligne IN3  IN0
mat_in5	: 1	;   //  INF + ligne IN3  IN0
mat_in6	: 1	;   //  ING + ligne IN3  IN0
mat_in7	: 1	;   //  INH + ligne IN3  IN0
mat_in8	: 1	;   //  INI + ligne IN3  IN0
;
;
; -------VARIABLES DE CALCULS INTERNES AU PIC -----------
indicateurs	: 1	; bit(0) = selection carte aprs rcp.code 0xC1, passe  1 (avec C = n de carte),
cpt		: 1	; Compteur incrment  chaque interruption du Timer0
cpt2		: 1	; Compteur incrment tous les 256 interruptions du Timer0 (cpt)
rs_adr	: 1	; Variable = adresse du registre  lire/crire = 0x20, 0x21, 0x22,,,
rs_mode	: 1	; Type de traitement en rception d'un caractre, bit(7)= cmd / donnes
pwm_cpt	: 1	;
out_out1	: 1	; Valeur calcule dans le PIC des bits des leds 1   7 et validation des 74HCT4094
out_out2	: 1	; Valeur calcule dans le PIC des bits des leds 8  11 + colonnes A  D (IC13)
out_pwm1	: 1	;| Sens_Inv(3) | Enable_Alim(2) | Sens_Inv(1) | Enable_Alim(0) | Enable_Alim(3) | Sens_Normal(2) | Enable_Alim(1) | Sens_Normal(0) | IC9  Alimentation des cantons
out_pwm2	: 1	;| Sens_Inv(7) | Enable_Alim(6) | Sens_Inv(5) | Enable_Alim(4) | Enable_Alim(7) | Sens_Normal(6) | Enable_Alim(5) | Sens_Normal(4) | IC10 (x)= n du canton
out_pwm3	: 1	;| Sens_Inv(11)| Enable_Alim(10)| Sens_Inv(9) | Enable_Alim(8) | Enable_Alim(11)| Sens_Normal(10)| Enable_Alim(9) | Sens_Normal(8) | IC11
out_pwm4	: 1	;| Sens_Inv(15)| Enable_Alim(14)| Sens_Inv(13)| Enable_Alim(12)| Enable_Alim(15)| Sens_Normal(14)| Enable_Alim(13)| Sens_Normal(12)| IC12
cpt_srv	: 1	; Numro de servo actif bits(3,2,1) +  bit indicateur bit(0) =
var1_spg	: 1	; Variable temporaire n1 utilisable dans tous les sous-programmes
var2_spg	: 1	; Variable temporaire n2 utilisable dans tous les sous-programmes
it_err	: 1	; Si le Timer0 TMR0 a dbord c'est une erreur, (it_err++)
libre1	: 1	; !!!! REGISTRE LIBRE  !!!!
libre2	: 1	; !!!! REGISTRE LIBRE  !!!!
libre3	: 1	; !!!! REGISTRE LIBRE  !!!!
libre4	: 1	; !!!! REGISTRE LIBRE  !!!!
	ENDC                   ; Fin de la zone


; Donnes en BANK1 de 0xA0  0xEF (80 adresses) car plus de place en BANK0
	CBLOCK	0xA0	;    Dbut de la zone variables en BANK1
det_i1_1	:1	; Octet de dtection des cantons 0  7
det_i1_2	:1	;     // coup d'avant
det_i1_3	:1	;     // coup d'avant - avant
det_i1_4	:1	;
det_i1_5	:1	;
det_i1_6	:1	;
;
det_i2_1	:1	; Octet de dtection des cantons 8  15
det_i2_2	:1	;     // coup d'avant
det_i2_3	:1	;     // coup d'avant - avant
det_i2_4	:1	;
det_i2_5	:1	;
det_i2_6	:1	;

val_srv	: 1	; Valeur du servo (0-255) selectionn
val_srvL	: 1	; Valeur du servo converti en 16 bits sur CCP1H/L
val_srvH	: 1	;

	ENDC                   ; Fin de la zone

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
; Dmarrage du programme  la mise sous tension.
		org		 	0x2100    ; Initialize EEPROM Data
		dt 		 	"UTS_2000_V34_ADR0 "	; Textes visibles dans le '.hex'.
if type_servo == 1 
		dt 		 	"Servo=1,00 msec a 2,00 msec"
endif
if type_servo == 2
		dt 		 	"Servo=0,88 msec a 2,11 msec"
endif
if type_servo == 3 
		dt 		 	"Servo=0,78 msec a 2,21 msec"
endif
if type_servo == 4
		dt 		 	"Servo=0,68 msec a 2,31 msec"
endif
if type_servo == 5
		dt 		 	"Servo=0,57 msec a 2,41 msec"
endif
		dt 		 	"Le 15/02/2020 "	
		dt 			"http://www.la-tour.info/uts/uts_index.html"	

		
		
		
		ORG   	 	 0		; 1 - Dmarrage  la mise sous tension
		goto   	 	init0
		ORG   	  	4		; 2 - En cas d'interruption
		goto    	it_timer
		dt 			"__UTS_2000_V33_ADR0__"	; Texte visible dans le .hex dans ICPROG
		dt			"__UTS_2000_V34_ADR0__"	; Textes visibles dans le '.hex'.
		dt 			"__Le 15/02/2020___"	
		dt 			"__http://www.la-tour.info/uts/uts_index.html___"	
;
init0	bsf	PCLATH,3	; Prpare le saut en page n0 vers n1 (0x800 - 0xFFF)
		call	init		; Saut en page n 1
		bcf	PCLATH,3	; On reste en page n0 (0x000 - 0x7FF) pour les CALL !
tdf	nop
		goto	tdf		; Boucle en page n0 en attendant les interruptions


;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
; Programme de gestion des interruptions
;
; Timer0 = 250 s ( 20 Mhz = 1250 instructions de 0,2 s)
; Timer0 = 250 s,(Pas du Timer0 = 20 Mhz/4 + prdiviseur  16 = 3,2 s)
;
; A 38400 Bd, le temps de transmission d'un caractre = 260 s
; Le timer TMR0 doit tre infrieur  ce temps. TMR0 = 250 s.
; Initialisation du Timer0 pour 250 s / 3,2 s = 78,1   ===>   256 - 78 = 178
; On prend 179 pour Timer0 < 250 s.
;
; cpt++ tous les 250 s et cpt2++ tous les 64 ms  20 Mhz.
;
it_timer
	bcf     INTCON,GIE	; Inhibe toutes les interruptions
;
; Pas de sauvegarde des registres, car le programme principal ne fait rien
; cela conomise des variables.
;
	bcf     INTCON,T0IF	; Remise  0 du drapeau de dpassement du Timer0 pour la prochaine interrup.
	movlw   .179		; dure = 0xB3 (255-XXXX)*temps_us du Timer0
	movwf   TMR0		; Interruption toutes les xxx instructions
	incf    cpt,f		; cpt  = Compteur incrment  chaque interruption du Timer0 (250 s)
	btfsc   STATUS,Z
	incf    cpt2,f		; cpt2 = Compteur incrment tous les 256 interruptions du Timer0

	call    cal_pwm		; Calculs des sorties PWM
	call    maj_pwm		; Sorties PWM vers 4094
	call    maj_rs232		; Liaison RS232

	btfss   cpt,2		; rpartiteur de charge suivant la valeur de cpt
	goto    cpt_0xx
cpt_1xx btfss   cpt,1
	goto    cpt_10x
cpt_11x btfss   cpt,0
	goto    cpt_6		; cpt = xxxx x110 = 6

	goto    cpt_7		; cpt = xxxx x111 = 7
cpt_10x btfss   cpt,0
	goto    cpt_4		; cpt = xxxx x100 = 4
	goto    cpt_5		; cpt = xxxx x101 = 5
cpt_0xx btfss   cpt,1
	goto    cpt_00x
cpt_01x btfss   cpt,0
	goto    cpt_2		; cpt = xxxx x010 = 2
	goto    cpt_3		; cpt = xxxx x011 = 3
cpt_00x btfss   cpt,0
	goto    cpt_0		; cpt = xxxx x000 = 0
	goto    cpt_1		; cpt = xxxx x001 = 1

cpt_0	call    maj_servo		; Calcul le prochain n de servo
	call    maj_out6		; Envoi des bits des sorties systme du PIC (servo,,,)
	goto    cpt_f

cpt_1	call    cal_led		; Calcul des octets  envoyer en sortie (leds multiplexes)
	call    maj_out5		; Envoi des bits des sorties  16  23  (IC16-ULN2008)
	goto    cpt_f

cpt_2	call    maj_can1		; Lecture ADRESH + choix du canal + lancement du convertisseur, bit ADCON0,ON = 1
	call    maj_out4		; Envoi des bits des sorties  8  15  (IC15-ULN2008)
	goto    cpt_f

cpt_3	call    maj_sig		; Clignotement de la led jaune
	call    maj_out3		; Envoi des bits des sorties  1  7  (IC14-ULN2008)
	goto    cpt_f

cpt_4	call    maj_can2		; Dmarrage de la conversion A/N, et lecture des 5 potentiomtres
	call    maj_out2		; Envoi des bits des leds  8  11 et A  D (IC13-ULN2008)
	goto    cpt_f

cpt_5	call    maj_out1		; Envoi des bits des leds  1  7 et validation des 74HCT4094
	goto    cpt_f

cpt_6	call    maj_in		; Lecture de la matrice d'entre des contacts
	goto    cpt_f

cpt_7	call    cal_servo		; Calcul le crneau du servo en cours 

	goto    cpt_f

cpt_f	btfsc   INTCON,T0IF	; Le timer  dj fait un tour ?
	incf    it_err,f		; Si le Timer0 'TMR'0 a dbord, c'est une erreur, (it_err++)
;
; Pas de restauration des registres, car le programme principal ne fait rien
;
	bsf     INTCON,GIE	; Autorise toutes les interruptions
	retfie


;-----------------------------------------------------------------------------
; SSPG_TOUJOURS : Calcul des valeurs des octets 'out_pwm1'  'out_pwm4'  envoyer
;                 directement pour commander les alimentations des cantons (L293D)
;                 Convertit les donnes du PC :
;                   - Vitesse = 'pwm_r0'      'pwm_r15'
;                   - Sens    = 'pwm_s0_7' et  'pwm_s8_15'
;
; Valeurs calcules dans le PIC pour la commande des L293D (Alimentation des cantons + Sens d'alimentation)
; Le train va dans une direction donne pour "Sens_Normal(x)=0" ou "Sens_Inv(x)=1" 
; Le train va dans l'autre direction pour    "Sens_Normal(x)=1" ou "Sens_Inv(x)=0" (Les pistes de certains cantons sont croises sur le circuit imprim).
; Cette configuration bizarre permet d'avoir sur le bornier de sorties des voies = 3-,3+,  2-,2+,  1-,1+,  0-,0+
; Les L293D alimentent les voies si "Enable_Alim(x)=1".      (x)= Numro du cantons.
; Sorties 74HC4094=Bits 7               6              5               4                3              2                  1              0
;   out_pwm1 : 1 ;| Sens_Inv(3) | Enable_Alim(2) | Sens_Inv(1) | Enable_Alim(0) | Enable_Alim(3) | Sens_Normal(2) | Enable_Alim(1) | Sens_Normal(0) | IC9 (x)= n du canton
;   out_pwm2 : 1 ;| Sens_Inv(7) | Enable_Alim(6) | Sens_Inv(5) | Enable_Alim(4) | Enable_Alim(7) | Sens_Normal(6) | Enable_Alim(5) | Sens_Normal(4) | IC10
;   out_pwm3 : 1 ;| Sens_Inv(11)| Enable_Alim(10)| Sens_Inv(9) | Enable_Alim(8) | Enable_Alim(11)| Sens_Normal(10)| Enable_Alim(9) | Sens_Normal(8) | IC11
;   out_pwm4 : 1 ;| Sens_Inv(15)| Enable_Alim(14)| Sens_Inv(13)| Enable_Alim(12)| Enable_Alim(15)| Sens_Normal(14)| Enable_Alim(13)| Sens_Normal(12)| IC12
;
; Bits de out_pwm(x) :  7  6  5  4  3  2  1  0
;          Bits(70) =  S  E  S  E  E  S  E  S  (Sens et Enable)
;                       3  2  1  0  3  2  1  0  (n de cantons)
;                       7  6  5  4  7  6  5  4  (n de cantons)
;                      15 14 13 12 15 14 13 12  (n de cantons)
;
; Cette configuration permet d'avoir sur le bornier de sorties des voies = 3-,3+,  2-,2+,  1-,1+,  0-,0+
;
; pwm_cpt = compteur de 0  65 aprs incrmentation au dbut de ce sspg
;           Les pas 0 et 65 servent au test d'occupation des cantons.  
; pwm_r0  = vitesse du canton de 0  64 = "PAS_VITESSE"
;
;            pwm_cpt = 01   02   03   04   05 -------- 61   62   63   64   65   00
; pwm_r0 =  0    out =  0    0    0    0    0           0    0    0    0    0    0
; pwm_r0 =  1    out =  1    0    0    0    0           0    0    0    0    0    0
; pwm_r0 =  2    out =  1    1    0    0    0           0    0    0    0    0    0
; pwm_r0 =  3    out =  1    1    1    0    0           0    0    0    0    0    0
; pwm_r0 =  4    out =  1    1    1    1    0           0    0    0    0    0    0
;...
; pwm_r0 = 62    out =  1    1    1    1    1           1    1    0    0    0    0
; pwm_r0 = 63    out =  1    1    1    1    1           1    1    1    0    0    0
; pwm_r0 = 64    out =  1    1    1    1    1           1    1    1    1    0    0
;                                                                           ^    ^      ^ = Test d'occupation des cantons.   
;
; Si pwm_cpt = 65 ou 0, test de l'occupation des 16 cantons = Sorties  haute impdance,
; pour faire fonctionner le dtecteur de tension.
;
; Inverse le bit de sens des cantons 1,3,5,7,9,11,13 et 15 = bit(5) et bit(7), car les pistes se croisent sur le circuit imprim.
; Le bit d'alimentation = bit(4), bit(1), bit(6) et bit(3) = bit 'Enable = ALIMENTATION des VOIES'.
;
cal_pwm
	incf    pwm_cpt,f
	movf    pwm_cpt,w
	sublw   PAS_VITESSE+2	; = nombre de pas de vitesse + 2 pas de test d'occupation de canton.
	btfsc   STATUS,Z
	clrf    pwm_cpt		; pwm_cpt = 0  65("PAS_VITESSE" + 1)
;
; Si pwm_cpt= 65 ou 0, alors on test la dtection des cantons
; Deux tests  la fin de chaque crneau  T0+250 s et T0+500S
; pour tre sr de dtecter un moteur mme avec un effet inductif important.
;
;
; Vitesses des cantons de 0  3
; Vitesses des              cantons(0),  cantons(1),  cantons(2),  cantons(3)  se traduit par 
; les bits d'alimentation = out_pwm1(4), out_pwm1(1), out_pwm1(6), out_pwm1(3) Si bit=1 le L293D alimente les voies.
;
	clrf    out_pwm1		; out_pwm1 = B'00000000'
	movf    pwm_r0,w		; cpt-cmd  f-w  0-2>C=1, 2-0>C=0, 0-0>C=0 cmd>cpt C=1
	subwf   pwm_cpt,w		; si (pwm_cpt = pwm_rx)
	btfss   STATUS,C		; Envoie donc un crneau en sortie  1 tant que vitesse du canton >  pwm_cpt
	bsf     out_pwm1,4	; Enable L293D = bits de sorties  =  4,  1,  6,  3

	movf    pwm_r1,w
	subwf   pwm_cpt,w
	btfss   STATUS,C
	bsf     out_pwm1,1

	movf    pwm_r2,w
	subwf   pwm_cpt,w
	btfss   STATUS,C
	bsf     out_pwm1,6

	movf    pwm_r3,w
	subwf   pwm_cpt,w
	btfss   STATUS,C
	bsf     out_pwm1,3
;
; Vitesses des cantons de 4  7
; Vitesses des              cantons(4),  cantons(5),  cantons(6),  cantons(7)  se traduit par 
; les bits d'alimentation = out_pwm2(4), out_pwm2(1), out_pwm2(6), out_pwm2(3) Si bit=1 le L293D alimente les voies.
;
	clrf    out_pwm2		; out_pwm2 = B'00000000'
	movf    pwm_r4,w		; cpt-cmd  f-w  0-2>C=1, 2-0>C=0, 0-0>C=0 cmd>cpt C=1
	subwf   pwm_cpt,w		; si (pwm_cpt = pwm_rx)
	btfss   STATUS,C		; Envoie donc un crneau en sortie  1 tant que vitesse du canton >  pwm_cpt
	bsf     out_pwm2,4	; Enable L293D = bits de sorties  =  4,  1,  6,  3

	movf    pwm_r5,w
	subwf   pwm_cpt,w
	btfss   STATUS,C
	bsf     out_pwm2,1

	movf    pwm_r6,w
	subwf   pwm_cpt,w
	btfss   STATUS,C
	bsf     out_pwm2,6

	movf    pwm_r7,w
	subwf   pwm_cpt,w
	btfss   STATUS,C
	bsf     out_pwm2,3
;
; Vitesses des cantons de 8  11
; Vitesses des              cantons(8),  cantons(9),  cantons(10), cantons(11) se traduit par 
; les bits d'alimentation = out_pwm3(4), out_pwm3(1), out_pwm3(6), out_pwm3(3) Si bit=1 le L293D alimente les voies.
;
	clrf    out_pwm3		; out_pwm3 = B'00000000'
	movf    pwm_r8,w		; cpt-cmd  f-w  0-2>C=1, 2-0>C=0, 0-0>C=0 cmd>cpt C=1
	subwf   pwm_cpt,w		; si (pwm_cpt = pwm_rx)
	btfss   STATUS,C		; Envoie donc un crneau en sortie  1 tant que vitesse du canton >  pwm_cpt
	bsf     out_pwm3,4	; Enable L293D = bits de sorties  =  4,  1,  6,  3

	movf    pwm_r9,w
	subwf   pwm_cpt,w
	btfss   STATUS,C
	bsf     out_pwm3,1

	movf    pwm_r10,w
	subwf   pwm_cpt,w
	btfss   STATUS,C
	bsf     out_pwm3,6

	movf    pwm_r11,w
	subwf   pwm_cpt,w
	btfss   STATUS,C
	bsf     out_pwm3,3
;
; Vitesses des cantons de 12  15
; Vitesses des              cantons(12), cantons(13), cantons(14), cantons(15) se traduit par 
; les bits d'alimentation = out_pwm4(4), out_pwm4(1), out_pwm4(6), out_pwm4(3) Si bit=1 le L293D alimente les voies.
;
	clrf    out_pwm4		; out_pwm4 = B'00000000'
	movf    pwm_r12,w		; cpt-cmd  f-w  0-2>C=1, 2-0>C=0, 0-0>C=0 cmd>cpt C=1
	subwf   pwm_cpt,w		; si (pwm_cpt = pwm_rx)
	btfss   STATUS,C		; Envoie donc un crneau en sortie  1 tant que vitesse du canton >  pwm_cpt
	bsf     out_pwm4,4	; Enable L293D = bits de sorties  =  4,  1,  6,  3

	movf    pwm_r13,w
	subwf   pwm_cpt,w
	btfss   STATUS,C
	bsf     out_pwm4,1

	movf    pwm_r14,w
	subwf   pwm_cpt,w
	btfss   STATUS,C
	bsf     out_pwm4,6

	movf    pwm_r15,w
	subwf   pwm_cpt,w
	btfss   STATUS,C
	bsf     out_pwm4,3
;
; Dfinit le sens d'alimentation des cantons 0  15.  Le bit de sens = "Sens_Inv(x)" ou "Sens_Normal(x)".
; Pour les cantons 0,2,4,6,8,10,12 et 14 la direction normale est quand le bit "Sens_Normal(x)" = 0
; Pour les cantons 1,3,5,7,9,11,13 et 15 la direction normale est quand le bit "Sens_Inverse(x)" = 1
; Inverse ou non le bit de sens de circulation reue dans la trame srie.
;
	btfsc   pwm_s0_7,0	; Bits de sens de circulation en entre de L293D.
	bsf     out_pwm1,0	; Sens_Normal(0)  -> out_pmw1(0) 
	btfss   pwm_s0_7,1
	bsf     out_pwm1,5	; Sens_Inverse(1) -> out_pmw1(5) 
	btfsc   pwm_s0_7,2
	bsf     out_pwm1,2	; Sens_Normal(2)  -> out_pmw1(2) 
	btfss   pwm_s0_7,3
	bsf     out_pwm1,7	; Sens_Inverse(3) -> out_pmw1(7) 
;
	btfsc   pwm_s0_7,4	; Bits de sens de circulation en entre de L293D.
	bsf     out_pwm2,0	; Sens_Normal(4)  -> out_pmw2(0)
	btfss   pwm_s0_7,5
	bsf     out_pwm2,5	; Sens_Inverse(5) -> out_pmw2(5)
	btfsc   pwm_s0_7,6
	bsf     out_pwm2,2	; Sens_Normal(6)  -> out_pmw2(2)
	btfss   pwm_s0_7,7
	bsf     out_pwm2,7	; Sens_Inverse(7) -> out_pmw2(7)
;
	btfsc   pwm_s8_15,0	; Bits de sens de circulation en entre de L293D.
	bsf     out_pwm3,0	; Sens_Normal(8)  -> out_pmw3(0)
	btfss   pwm_s8_15,1
	bsf     out_pwm3,5	; Sens_Inverse(9) -> out_pmw3(5)
	btfsc   pwm_s8_15,2
	bsf     out_pwm3,2	; Sens_Normal(10) -> out_pmw3(2)
	btfss   pwm_s8_15,3
	bsf     out_pwm3,7	; Sens_Inverse(11)-> out_pmw3(7)
;
	btfsc   pwm_s8_15,4	; Bits de sens de circulation en entre de L293D.
	bsf     out_pwm4,0	; Sens_Normal(12) -> out_pmw4(0)
	btfss   pwm_s8_15,5
	bsf     out_pwm4,5	; Sens_Inverse(13)-> out_pmw4(5)
	btfsc   pwm_s8_15,6
	bsf     out_pwm4,2	; Sens_Normal(14) -> out_pmw4(2)
	btfss   pwm_s8_15,7
	bsf     out_pwm4,7	; Sens_Inverse(15)-> out_pmw4(7)

	return


;-----------------------------------------------------------------------------
; SSPG_TOUJOURS : Mise  jour des sorties PMW, sortie et validation des bits
;                 dans les 74HCT4094 en fonction des octets 'out_pwm1'  'out_pwm4'
;                 calculs prcdemment
;
; Lecture simultane des entres de dtections des cantons (74HCT165):
; on attend 2 * 250 s avant de lire les bits d'occupation.
; Pour amliorer la fiabilit de la dtection, il faudrait essayer de lire aux deux
; pas de pause et faire la moyenne de dtection sur 2x3 mesures.
;
; Pour simplifier le circuit imprim, les bits sont rpartis de la manire suivante :
;
;   Bits lus        :  0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15
;   Cantons         :  3,  1, 15, 13, 11,  9,  7,  5,  4,  6,  8, 10, 12, 14,  0,  2
;   var1_spg/n bit :  3   1                   7   5   4   6                   0   2
;   var2_spg/n bit :          7   5   3   1                   0   2   4   6
; <=>
;   Cantons         :  0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15
;   Bits lus        : 14,  1, 15,  0,  8,  7,  9,  6, 10,  5, 11,  4, 12,  3, 13,  2;
;
maj_pwm
	bcf	PORTB,LODDEC2	; Charge les entres  chaque fois
	clrf	var1_spg		; Valeur des bits de dtection d'occupation
	clrf	var2_spg		; Valeur des bits de dtection d'occupation
	nop				; mais les donnes ne seront traites qu'une fois sur 65
	bsf	PORTB,LODDEC2	; Impulsion de validation de 800 ns
	nop
	nop

	btfsc PORTB,DATDEC2	; Lecture simultane des 74HCT165 d'occupation des cantons
	bsf	var1_spg,3
	btfss out_pwm4,7
	CK_DOWN_PWM
	btfsc out_pwm4,7
	CK_UP_PWM

	btfsc PORTB,DATDEC2
	bsf	var1_spg,1
	btfss out_pwm4,6
	CK_DOWN_PWM
	btfsc out_pwm4,6
	CK_UP_PWM

	btfsc PORTB,DATDEC2
	bsf	var2_spg,7
	btfss out_pwm4,5
	CK_DOWN_PWM
	btfsc out_pwm4,5
	CK_UP_PWM

	btfsc PORTB,DATDEC2
	bsf	var2_spg,5
	btfss out_pwm4,4
	CK_DOWN_PWM
	btfsc out_pwm4,4
	CK_UP_PWM

	btfsc PORTB,DATDEC2
	bsf	var2_spg,3
	btfss out_pwm4,3
	CK_DOWN_PWM
	btfsc out_pwm4,3
	CK_UP_PWM

	btfsc PORTB,DATDEC2
	bsf	var2_spg,1
	btfss out_pwm4,2
	CK_DOWN_PWM
	btfsc out_pwm4,2
	CK_UP_PWM

	btfsc PORTB,DATDEC2
	bsf	var1_spg,7
	btfss out_pwm4,1
	CK_DOWN_PWM
	btfsc out_pwm4,1
	CK_UP_PWM

	btfsc PORTB,DATDEC2
	bsf	var1_spg,5
	btfss out_pwm4,0
	CK_DOWN_PWM
	btfsc out_pwm4,0
	CK_UP_PWM

	btfsc PORTB,DATDEC2
	bsf	var1_spg,4
	btfss out_pwm3,7
	CK_DOWN_PWM
	btfsc out_pwm3,7
	CK_UP_PWM

	btfsc PORTB,DATDEC2
	bsf	var1_spg,6
	btfss out_pwm3,6
	CK_DOWN_PWM
	btfsc out_pwm3,6
	CK_UP_PWM

	btfsc PORTB,DATDEC2
	bsf	var2_spg,0
	btfss out_pwm3,5
	CK_DOWN_PWM
	btfsc out_pwm3,5
	CK_UP_PWM

	btfsc PORTB,DATDEC2
	bsf	var2_spg,2
	btfss out_pwm3,4
	CK_DOWN_PWM
	btfsc out_pwm3,4
	CK_UP_PWM

	btfsc PORTB,DATDEC2
	bsf	var2_spg,4
	btfss out_pwm3,3
	CK_DOWN_PWM
	btfsc out_pwm3,3
	CK_UP_PWM

	btfsc PORTB,DATDEC2
	bsf	var2_spg,6
	btfss out_pwm3,2
	CK_DOWN_PWM
	btfsc out_pwm3,2
	CK_UP_PWM

	btfsc PORTB,DATDEC2
	bsf	var1_spg,0
	btfss out_pwm3,1
	CK_DOWN_PWM
	btfsc out_pwm3,1
	CK_UP_PWM

	btfsc PORTB,DATDEC2
	bsf	var1_spg,2
	btfss out_pwm3,0
	CK_DOWN_PWM
	btfsc out_pwm3,0
	CK_UP_PWM

	btfss out_pwm2,7
	CK_DOWN_PWM
	btfsc out_pwm2,7
	CK_UP_PWM

	btfss out_pwm2,6
	CK_DOWN_PWM
	btfsc out_pwm2,6
	CK_UP_PWM

	btfss out_pwm2,5
	CK_DOWN_PWM
	btfsc out_pwm2,5
	CK_UP_PWM

	btfss out_pwm2,4
	CK_DOWN_PWM
	btfsc out_pwm2,4
	CK_UP_PWM

	btfss out_pwm2,3
	CK_DOWN_PWM
	btfsc out_pwm2,3
	CK_UP_PWM

	btfss out_pwm2,2
	CK_DOWN_PWM
	btfsc out_pwm2,2
	CK_UP_PWM

	btfss out_pwm2,1
	CK_DOWN_PWM
	btfsc out_pwm2,1
	CK_UP_PWM

	btfss out_pwm2,0
	CK_DOWN_PWM
	btfsc out_pwm2,0
	CK_UP_PWM

	btfss out_pwm1,7
	CK_DOWN_PWM
	btfsc out_pwm1,7
	CK_UP_PWM

	btfss out_pwm1,6
	CK_DOWN_PWM
	btfsc out_pwm1,6
	CK_UP_PWM

	btfss out_pwm1,5
	CK_DOWN_PWM
	btfsc out_pwm1,5
	CK_UP_PWM

	btfss out_pwm1,4
	CK_DOWN_PWM
	btfsc out_pwm1,4
	CK_UP_PWM

	btfss out_pwm1,3
	CK_DOWN_PWM
	btfsc out_pwm1,3
	CK_UP_PWM

	btfss out_pwm1,2
	CK_DOWN_PWM
	btfsc out_pwm1,2
	CK_UP_PWM

	btfss out_pwm1,1
	CK_DOWN_PWM
	btfsc out_pwm1,1
	CK_UP_PWM

	btfss out_pwm1,0
	CK_DOWN_PWM
	btfsc out_pwm1,0
	CK_UP_PWM

	nop
	nop
	bsf   PORTB,LODPWM	; Charge les donnes dans les 74HCT4094
	nop
	nop
	nop
	bcf   PORTB,LODPWM	; Impulsion de validation de 800 ns
;
; Traitement des bits d'occupation 'var1_spg1' et 'var1_spg2' en V3.2
; quand 'pwm_cpt' = 0 ou 'PAS_VITESSE' + 1
; Deux tests x 3 priodes = 6 variables pour l'historique.
; Si le canton est occup alors sa valeur = 1
; Les calculs se font en BANK1, car il n'y a plus de place en BANK0
; Si le dtecteur est trop sensible, il faut supprimer la dtection 
; du premier pas quand pwm_cpt= "PAS_VITESSE" + 1.
;
	movf	pwm_cpt,w
	sublw	PAS_VITESSE+1
	btfsc	STATUS,Z
	goto	dect			; pwm_cpt = 65 + "PAS_VITESSE" + 1

	movf	pwm_cpt,w
	btfss	STATUS,Z
	goto	att_pwm		; pwm_cpt # 0
;
; Cela fait 1 ou 2 x TMR0 = 250 ou 500 s que les sorties PWM sont inhibes,
; pour que la dtection d'occupation des cantons soit efficace.
; Le rsultat est dans 'var1_spg' et 'var2_spg'.
;
dect	BANK1
	movf	det_i1_5,w		; '00000001' = canton '1' occup
	movwf	det_i1_6		; Dcalage des historiques
	movf	det_i1_4,w
	movwf	det_i1_5
	movf	det_i1_3,w
	movwf	det_i1_4
	movf	det_i1_2,w
	movwf	det_i1_3		; 5->6,  4->3,  3->4
	movf	det_i1_1,w		; 2->3,  1->2,  nouveau->1
	movwf	det_i1_2
	BANK0
	movfw	var1_spg
	BANK1
	movwf	det_i1_1

	iorwf	det_i1_2,w		; Calcul de l'tat d'occupation sur 
	iorwf	det_i1_3,w		; les 6 derniers tats
	iorwf	det_i1_4,w
	iorwf	det_i1_5,w
	iorwf	det_i1_6,w
	BANK0
	movwf	det_i1		; Etat d'occupation sur les 2x3 derniers tests
	iorwf	det_i1m,f		; Etat mmoris

	BANK1
	movf	det_i2_5,w		; '00000001' = canton '1' occup
	movwf	det_i2_6		; Dcalage des historiques
	movf	det_i2_4,w
	movwf	det_i2_5
	movf	det_i2_3,w
	movwf	det_i2_4
	movf	det_i2_2,w
	movwf	det_i2_3		; 5->6,  4->3,  3->4
	movf	det_i2_1,w		; 2->3,  1->2,  nouveau->1
	movwf	det_i2_2
	BANK0
	movfw	var2_spg
	BANK1
	movwf	det_i2_1

	iorwf	det_i2_2,w		; Calcul de l'tat d'occupation
	iorwf	det_i2_3,w		; les 6 derniers tats
	iorwf	det_i2_4,w
	iorwf	det_i2_5,w
	iorwf	det_i2_6,w
	BANK0
	movwf	det_i2		; Etat d'occupation sur les 2x3 derniers tests
	iorwf	det_i2m,f		; Etat mmoris
	goto	fin_pwm
	
att_pwm				; Retard si pas de calcul de bit d'occupation pour 
	movlw	.15			; viter un dcalage  l'appel de la routine rs232
	movwf	var1_spg		; il faut perdre des instructions ici
bcl_pwm				; pour garder une temps constant dans 'dect'
	decfsz var1_spg,f		; dcrmenter var1_spg
	goto	bcl_pwm		; si pas 0, boucler	
	goto	fin_pwm

fin_pwm
	return

;-----------------------------------------------------------------------------
; SSPG_TOUJOURS : Liaison srie, interprtation des caractres reus
;
; Les lignes suivantes dcrivent le protocole utilis :
; 0 : synchronisation : en envoyant la commande 0x00, le UC va se synchroniser, c'est utile lorsqu'on se perd dans le protocole.
;     Un envoie de quelques 0 devrait permettre de rcuprer toutes les erreurs de protocole.
; 1 : Test : en envoyant la commande 0xy1, le UC devrait rpondre 0x83 (si la carte  l'adresse = y-1).
;     Cette commande peut servir  vrifier que le moniteur est bien actif et qu'il est possible de communiquer avec le UC.
;     Si la carte rpond 0x83, c'est que la carte  l'adresse 0.
;     Si la carte rpond 0x83, elle est maintenant slectionne et les commandes suivantes lui sont destines.
; 2 : Adresse : en envoyant la commande 0x02 puis l'adresse d'un registre.
;     On spcifie l'adresse du registre  utiliser pour les oprations suivantes de lecture et d'criture.
;     L'adresse tant sur 8 bits, il est possible d'accder aux banques 0 et 1.
; 3 : Commande d'criture dans un registre
;     En envoyant la commande 0x03 puis un octet  crire, cet octet est mis dans le registre slectionn.
; 4 : Commande de lecture dans un registre
;     En envoyant la commande 0x04, la carte envoie au PC la valeur courante du registre slectionn.
; 5 : Ecriture incrmentale
;     La commande 0x05 suivit d'un octet, permet de passer au registre suivant,
;     en incrmentant l'adresse courante, puis d'y crire cet octet.
; 6 : Lecture incrmentale
;     La commande x06 permet de passer au registre suivant en incrmentant
;     l'adresse courante, puis de renvoyer la valeur de ce registre.
; 7 : Lecture puis effacement
;     En envoyant la commande 0x07, le moniteur renvoie la valeur courante
;     du registre slectionn et le met  0.
; 8 : Dclenche un transfert en lecture et criture de tous les registres couramment utiliss par UTS.
;     Ce mode fait gagner beaucoup de temps car il n'y a pratiquement que de la donne qui circule.
;     un octet [0x01], la carte rpond en envoyant 0x83
;     un octet [0x08], la carte se met en attente de donnes.
;     un octet pour [0x20]
;     un octet pour [0x21], la carte rpond en envoyant le contenu du register 0x4f (can_r0)
;     un octet pour [0x22]
;     un octet pour [0x23], la carte rpond en envoyant le contenu du register 0x50 (can_r1)
;     un octet pour [0x24]
;     un octet pour [0x25], la carte rpond en envoyant le contenu du register 0x4b (can_r2)
;     ...
;     un octet pour [0x4A],
;     un octet pour [0x4B],
;     un octet pour [0x4C],
;     un octet pour [0x4D],
;     un octet pour [0x53], plus de rponse et fin du transfert.
;
; 8x (hex) : slectionne une carte. (Les cartes avec l'ancienne version 873_rs11 flashe ne tiennent
;     pas compte de cette commande. Le soft sait aussi reconnatre la version de la carte.
;     Si une ancienne carte est utilise, elle sera considre comme la carte 0. Rappelons qu'il n'est pas possible
;     d'utiliser des anciennes et nouvelles cartes en mme temps. Soit une ancienne carte, soit de 1  4 nouvelles cartes.
;
; Le ping (1) permet galement de slectionner la carte suivant son adresse CARDADR (= 0,1,2 ou 3)
; En fonction de la carte, il faut mettre la commande ping suivante  :
; Adresse =         0     1     2     3
; Octet   =      0x01  0x11  0x21  0x31
; Touches = ALT + 001 + 017 + 033 + 049  (sous simulateur ISIS)
;
; rs_mode(7) : 0 = Attente d'un octet de commande / 1 = Attente d'un octet de donnes
;
; En V3.0  38400Bd, temps de rception d'un caractre = 260 s > Timer0 = 250 s
; Donc, on ne perd pas d'octet  la rception.
; En mission, on va envoyer un octet une fois sur deux = / 500 s pour tre sur que
; le dernier octet soit parti avant d'envoyer le prochain.
;
maj_rs232
	btfss   PIR1,RCIF		; le buffer de rception est-il plein ?
	return			; non ! = on arrte l
	movf    RCREG,w		; lire le registre de rception RCREG ce qui efface PIR1,RCIF
	movwf   var1_spg		; var1_spg = caractre reu

	btfsc   rs_mode,7		; Octet de donnes attendu ?
	goto    rs_l2y		; oui! => rs_12y
;
; rs_mode = B'0xxxxxxx' => On a reu un octet de commande
;
rs_l2n  btfsc   var1_spg,3
	goto    rs_8		; bit rs_mode(7) = 0, octet recu = "00001000" = 0x08
	btfss   var1_spg,2
	goto    rs_0xx
rs_1xx  btfss   var1_spg,1
	goto    rs_10x
rs_11x  btfss   var1_spg,0
	goto    rs_6		; bit rs_mode(7) = 0, octet recu = "00000110" = 0x06
	goto    rs_7		; bit rs_mode(7) = 0, octet recu = "00000111" = 0x07
rs_10x  btfss   var1_spg,0
	goto    rs_4		; bit rs_mode(7) = 0, octet recu = "00000100" = 0x04
	goto    rs_5		; bit rs_mode(7) = 0, octet recu = "00000101" = 0x05
rs_0xx  btfss   var1_spg,1
	goto    rs_00x
rs_01x  btfss   var1_spg,0
	goto    rs_2		; bit rs_mode(7) = 0, octet recu = "00000010" = 0x02
	goto    rs_3		; bit rs_mode(7) = 0, octet recu = "00000011" = 0x03
rs_00x  btfss   var1_spg,0
	goto    rs_0		; bit rs_mode(7) = 0, octet recu = "00000000" = 0x00
	goto    rs_1		; bit rs_mode(7) = 0, octet recu = "00000000" = 0x01
;
rs_1	bcf     indicateurs,rs_sel	; octet reu = 0x01, 0x11, 0x21 ou 0x31 => test de ping
	swapf   var1_spg,w		; Si l'adresse de la carte est dans l'octet,
	andlw   0x07			; on slectionne la carte
	sublw   CARDADR
	btfss   STATUS,Z		; btfss   STATUS,C modif FF
	goto    rs_fin
	bsf     indicateurs,rs_sel	; Adresse OK, => La carte est maintenant slectionne!
	movlw   0x83		; Octet de rponse au PC = 0x83
	call    tx
	goto    rs_fin

rs_2	movlw   0x81		; octet reu = 0x02, => octet pour signaler que le prochain octet
	movwf   rs_mode		; est  placer dans le registre 'FSR' (pointeur)
	goto    rs_fin

rs_3	movlw   0x82		; octet reu = 0x03, => octet pour signaler que le prochain octet
	movwf   rs_mode		; est  crire dans [FSR] (adresse pointe par FSR)
	goto    rs_fin

rs_5	movlw   0x84		; octet reu = 0x05, => octet pour signaler que le prochain octet
	movwf   rs_mode		; est  crire dans [FSR] (adresse pointe par FSR) et incrmenter 'FSR'
	goto    rs_fin

rs_6	btfss   indicateurs,rs_sel	; octet reu = 0x06, => octet pour signaler que le prochain octet
						; est  lire dans [FSR] (adresse pointe par FSR) et incrmenter 'FSR'
	goto    rs_fin		; Si la carte n'est pas slectionne => goto fin
	incf    rs_adr,f		; inc rs_adr = adresse du registre  lire and read

rs_4	btfss   indicateurs,rs_sel	; octet reu = 0x04, => octet pour signaler que le prochain octet
						; est  lire dans [FSR] (adresse pointe par FSR)
	goto    rs_fin		; Si la carte n'est pas slectionne => goto fin
	movf    rs_adr,w
	movwf   FSR
	movf    INDF,w
	call    tx			; envoi de [FSR] au PC (contenu point par 'FSR')
	goto    rs_fin

rs_7	btfss   indicateurs,rs_sel	; octet reu = 0x07, => octet pour signaler que le prochain octet
						; est  lire dans [FSR] puis on l'fface (adresse pointe par FSR)
	goto    rs_fin		; Si la carte n'est pas slectionne => goto fin
	incf    rs_adr,f
	movf    rs_adr,w		; inc rs_adr = adresse du registre  lire and read + clear
	movwf   FSR
	movf    INDF,w
	clrf    INDF
	call    tx			; envoi de [FSR] au PC (contenu point par 'FSR')
	goto    rs_fin

rs_8	movlw   0x88		; octet reu = 0x08, => octet pour signaler un dbut de transfert
	movwf   rs_mode		; automatique des donnes (lecture + criture)
	movlw   0x20-1		; PREMIER_ECR - 1
	movwf   rs_adr
	bcf     indicateurs,rs_env	; V3.0 On est prt  mettre un caractre une fois sur deux
					 ; mais il faut attendre que la rponse au 0x01 soit partie.

rs_0	goto    rs_fin		; octet reu = 0x00, => Ne fait rien, re-synchonisation ventuelle
;
; rs_mode = B'1xxxxxxx' => On a reu un octet de donnes
;
rs_l2y
	btfsc   rs_mode,3	;
	goto    rs_2_8
	bcf     rs_mode,7
	btfss   indicateurs,rs_sel	; Si la carte n'est pas slectionne => goto fin
	goto    rs_fin

	btfsc   rs_mode,0
	goto    rs_2_2
	btfsc   rs_mode,1
	goto    rs_2_3
	goto    rs_2_5
rs_2_2  movf    var1_spg,w		; set adr
	movwf   rs_adr
	goto    rs_fin
rs_2_5  incf    rs_adr,f	; inc adr and write
rs_2_3  movf    rs_adr,w	; write
	movwf   FSR
	movf    var1_spg,w
	movwf   INDF
	goto    rs_fin
;
; PREMIER_ECR = 0x20 = Adresse du premier octet en criture vers le PIC
; DERNIER_ECR = 0x53 = Adresse du dernier octet en criture vers le PIC
; PREMIER_LEC = 0x4F = Adresse du premier octet en lecture depuis le PIC
; PREMIER_LEC_EFF = 0x61 = Adresse du premier octet en lecture + ffacement depuis le PIC
; DERNIER_LEC = 0x6B = Adresse du dernier octet en lecture depuis le PIC
;
rs_2_8
	incf    rs_adr,f		; Global Transfert pointeur = rs_adr ='PREMIER_ECR' - 1 (au dbut)
	movlw   0x53		; 'DERNIER_ECR' = si dernire adresse d'criture dans registre
	subwf   rs_adr,w		; W = 255 - 'DERNIER_ECR' + rs_adr
	btfsc   STATUS,Z
	bcf     rs_mode,7		; ne plus excuter l2y, fin du mode donnes
	btfss   indicateurs,rs_sel	; Si la carte n'est pas slectionne => goto fin
	goto    rs_fin

	movf    rs_adr,w
	movwf   FSR			; Pointeur d'criture = [FSR]
	movf    var1_spg,w	; 1 / Ecrit les donnes recues de 'PREMIER_ECR'  'DERNIER_ECR'
	movwf   INDF		; Ecrit la donne reue  l'adresse pointe par [FSR] dans le PIC

	movlw   0x53-0x20		; 'DERNIER_ECR' - 'PREMIER_ECR'
	addwf   FSR,f		; pointeur de lecture = [FSR]
	movlw   0xFF-0x83		; 0xFF - 0x?? : car on n'envoi un octet une fois sur deux
	addwf   FSR,w
	btfsc   STATUS,C
	goto    rs_fin		; Si dernire adresse de lecture dans un registre

	btfsc   indicateurs,rs_env
	goto    c_env
;
; si  rs_env = 0  alors  rs_env = 1 + goto fin
; 'rs_env' permet d'envoyer un caractre au PC une fois sur deux, sinon on sature
; le buffer d'mission. A la premiere rception d'un caractre, on ne renvoi rien.
;
	bsf	  indicateurs,rs_env	; On n'enverra un caractre le prochain tour
	goto    rs_fin

c_env	bcf	  indicateurs,rs_env	; On n'enverra rien le prochain tour
	rrf     FSR,f		; divise par deux le pas
	movlw   0x2a		; 1er car.  envoyer = 'PREMIER_LEC'
	addwf   FSR,f
	movf    INDF,w		; 2 / Envoi les donnes au PC de ['PREMIER_LEC'  'DERNIER_LEC'
	call    tx			; 3 / Efface les donnes envoyes au PC de 'PREMIER_LEC_EFF'  'DERNIER_LEC'

	movlw   0xFF-0x61+1	; '0xFF' - 'PREMIER_LEC_EFF' + 1 = Derniers octets  ffacer
	addwf   FSR,w
	btfsc   STATUS,C
	clrf    INDF		; efface [FSR]
rs_fin
	return
;
; Caractre  envoyer vers le PC dans W
;
tx	BANK1				; passer en banque1
	btfss   TXSTA,TRMT	; attente que le dernier caractre soit parti
	goto    tx
	BANK0				; passer en banque0
	movwf   TXREG		; placer dans TXREG le caractre  envoyer
	return


;-----------------------------------------------------------------------------
; SSPG_CPT0 : Servos
; On calcul la valeur  envoyer sur la carte pour orienter le crneau vers le
; bon servo. L'aiguillage sera pris en compte quand cpt = 6 (validation des sorties).
; cpt_srv = B'0000XXXY' XXX = n servo actif, si Y=1 => envoyer le crneau en sortie.
;
; cpt=0 : On incrment cpt_srv, On envoie en sortie le n de servo actif = cpt_srv/2.
;
;             >---< = 2msec           >-------< = 4msec
; cpt =       0   0   0   0   0   0   0   0   0   0   0   0  => (sortie de cpt_srv/2)
; cpt_srv =   0,  1,  2,  3,  4,  5,  6,  7,  8,  9,  0,  1 
; n servo =  0       1       2       3       4       0
; cpt =           7       7       7       7       7       7  => (sortie du crneau de cpt_srv/2)
;
maj_servo
	incf    cpt_srv,f
	movfw   cpt_srv
	sublw   .10			; .10 = 5 servos maximum.
	btfsc   STATUS,Z
	clrf    cpt_srv		; cpt_srv = 0,1,2...,8,9, 0,1,2
; Ensuite dans 'maj_out6', on envoi les bits : cpt_srv(3), cpt_srv(2) et cpt_srv(1) sur les sorties.
; Le bit : cpt_srv(0)=0 sert  connaitre le dbut du crneau.
	return


;-----------------------------------------------------------------------------
; SSPG_CPT1 : Calcul du multiplexage des leds
;             Calcul des octets  envoyer en sortie = out_out1 et out_out2
;             A chaque passage, change le numro de colonne  activer = out_out2(4  7)
;
; LED 00-07 = led_on0  (allume) + led_cl0 (clignotante)  + led_in0 (clignotant invers)
; LED 08-12 = led_on01 (allume) + led_cl01 (clignotante) + led_in01 (clignotant invers) [QUARTE de DROITE]
; LED 13-20 = led_on1  (allume) + led_cl1 (clignotante)  + led_in1 (clignotant invers)
; LED 13-20 = led_on01 (allume) + led_cl01 (clignotante) + led_in01 (clignotant invers) [QUARTE de GAUCHE]
; ...
;
cal_led
	clrw
	btfsc		cpt2,3		; cpt2 = Compteur incrment tous les 256 interruptions du Timer0 (64 ms  20Mhz)
	movlw		0xff
	movwf		var1_spg		; var1_spg passe de 00  FF toutes les 1,6 secondes (comme la led jaune)
	btfsc		cpt,4
	goto		led_1X
led_0X  btfss	cpt,3
	goto		led_0
	goto		led_1
led_1X  btfss	cpt,3
	goto		led_2
	goto		led_3

led_0	movf    var1_spg,w		; cpt = B'---00---'
	xorwf   led_in0,w
	iorwf   led_cl0,w
	andwf   led_on0,w
	movwf   out_out1		; out_out1 = sortie leds 0  7

	movf    var1_spg,w
	xorwf   led_in01,w
	iorwf   led_cl01,w
	andwf   led_on01,w
	andlw   0x0f
	movwf   out_out2		; out_out2 = sortie leds 8  11 + slection colonne A
	bsf     out_out2,4
	goto    led_f

led_1	movf    var1_spg,w		; cpt = B'---01---'
	xorwf   led_in1,w
	iorwf   led_cl1,w
	andwf   led_on1,w
	movwf   out_out1
	movf    var1_spg,w
	xorwf   led_in01,w
	iorwf   led_cl01,w
	andwf   led_on01,w
	movwf   out_out2
	swapf   out_out2,w
	andlw   0x0f
	movwf   out_out2
	bsf     out_out2,5	; out_out2 = sortie leds 8  11 + slection colonne B
	goto    led_f

led_2	movf    var1_spg,w		; cpt = B'---10---'
	xorwf   led_in2,w
	iorwf   led_cl2,w
	andwf   led_on2,w
	movwf   out_out1
	movf    var1_spg,w
	xorwf   led_in23,w
	iorwf   led_cl23,w
	andwf   led_on23,w
	andlw   0x0f
	movwf   out_out2
	bsf     out_out2,6	; out_out2 = sortie leds 8  11 + slection colonne C
	goto    led_f

led_3	movf    var1_spg,w		; cpt = B'---11---'
	xorwf   led_in3,w
	iorwf   led_cl3,w
	andwf   led_on3,w
	movwf   out_out1
	movf    var1_spg,w
	xorwf   led_in23,w
	iorwf   led_cl23,w
	andwf   led_on23,w
	movwf   out_out2
	swapf   out_out2,w
	andlw   0x0f
	movwf   out_out2
	bsf     out_out2,7	; out_out2 = sortie leds 8  11 + slection colonne D
	goto    led_f
led_f	return


;-----------------------------------------------------------------------------
; SSPG_CPT2 : Lecture des 5 entres analogiques, une  la fois (sans utiliser les interruptions CAN)
;             Rsultat dans : can_r0, can_r1 ... can_r4
;             A 250 s lecture de toutes les entres analogiques en 16 ms = 60 Hz
;
;   maj_can1 = Lecture ADRESH + choix du canal + lancement du convertisseur, bit ADCON0,ON = 1
;              et attente de 23s minimum  20 Mhz
;   maj_can2 = Dmarrage de la conversion + bit ADCON0,GO_DONE = 1
;              et attente de la fin de la conversion
; ADCON0 :
;   ADCS10  =  10 = F conversion = Fosc/32 si Quartz > 5 Mhz
;   CHS210  = nnn = n du canal de conversion en cours
;   GO/DONE =   0 =  mettre  1 aprs une conversion
;   nc      =   0
;   ADON    =   1 = Mise en route du convertisseur
;
maj_can1				; Lecture ADRESH + choix canal + lancement convert, bit ADCON0,ON = 1
;
	movf		cpt,w		; cpt = '--yyy---' yyy = n de canal CAN
	andlw		B'00111000'
	sublw		B'00100000'
	btfss		STATUS,C
	return			; goto fin si yyy > 4

	rlf		cpt,w		; w   = '-yyy----' avec yyy = 0  4 !
	andlw		B'01110000' ; w   = '0yyy0000'
	movwf		FSR
	swapf		FSR,f		; FSR = '00000yyy'
	movlw		can_r0
	addwf		FSR,f		; FSR = adresse can_r0 + n du canal en cours de lecture = adresse can_rX
	movf		ADRESH,w	; Lecture de la conversion A/N sur un octet
	movwf		INDF		; Ecriture dans [INDF] = can-r0  can_r4
;
; Prparation de la prochaine conversion A/N
;
	movf		cpt,w		; w   = '--yyy---'
	andlw		B'00111000' ; w   = '00yyy000'
	xorlw		B'00100000'
	btfsc		STATUS,Z	; Si le canal = n4, le prochain canal sera le n 0
	movlw		B'11011000'
	xorlw		B'00100000'
	addlw		B'00001000'
	iorlw		B'10000001'
	movwf		ADCON0	; ADCON0 = '10xxx001' avec xxx = yyy + 1
	return


;-----------------------------------------------------------------------------
; SSPG_CPT3 : Clignotement de la led jaune
;
maj_sig
	btfss		cpt2,3	; clignotement de la led jaune sur (RA4) (1.92Hz)
	bcf		PORTA,4	; led jaune = on = 16 * 256 * 250 s
	btfsc		cpt2,3
	bsf		PORTA,4	; led jaune = off
	return


;-----------------------------------------------------------------------------
; SSPG_CPT4 : Prparation  la conversion A/N
;
maj_can2				; Dmarrage de la conversion, bit ADCON0,GO_DONE = 1
	bsf		ADCON0,GO_DONE
	return


;-----------------------------------------------------------------------------
; SSPG_CPT6 : Lecture de la matrice des contacts d'entre = INA  INB x 4 bits
; Attention en V3.1, la patte PORTC(2) sert  commander les servos..
; L'entre PORTC(2) est remplace par l'entre PORTB(0). = C3,B0,C1,C0
;
; Le 30/01/2015 : Nouvelle tempo en lecture du clavier. 
;
; Les 4 bits de droite sont automatiquement effacs juste aprs leur lecture.
; mat_in0 = 4 bits de la colonne INA + 4 bits du mme tat mmoris
; mat_in1 =  // INB
; mat_in2 =  // INC
; . . . .
; mat_in8 =  // INI
;
; En V3.0 Horloge = C(4) est passe en B(1). 
; Il faut remettre toute les sorties du 74HCT4094  '1' avant de commencer,
; car cette sortie est aussi utilise par d'autre fonctions.
;
maj_in
	bsf   PORTC,DATMAT	; En V3.0 pour retrouver les 9 sorties  1
	nop
	CK_UD_DAT
	CK_UD_DAT
	CK_UD_DAT

	CK_UD_DAT
	CK_UD_DAT
	CK_UD_DAT

	CK_UD_DAT
	CK_UD_DAT
	CK_UD_DAT

	bcf   PORTC,DATMAT
	nop
	bcf   PORTB,CLKMAT
	nop
	nop

	bsf   PORTB,CLKMAT	; Sortie (A) = 0 !
	nop
	nop                     ; Ajout de 10 "nop" en V3.3 le 20/02/2015 pour viter des erreurs de lecture
	nop                     ; en double d'un clavier multiplex.
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	movlw B'00001111'
	andwf mat_in0,f		; mat_in0 = "0000YYYY"  Y = ancien tat mmoris
	comf  PORTC,w		; w = inverse(PORTC), X = 1 si le contact est ferm
	bcf   PORTB,CLKMAT
	bsf   PORTC,DATMAT	; Remet la sortie  1 (Le crneau 0 en sortie dfile)
	andlw B'00001011'		; w = "0000X0XX", seul les bits C3, C1 et C0 sont utiliss. 
	btfss PORTB,0		; Le bit C2 est utilis pour la commande des servos
	iorlw B'00000100'		; Le bit C2 est remplac par l'entre B0
	iorwf mat_in0,f		; 'OU' mat_in0 OU (0000XXXX", X = 1 contact ferm) => mat_in0
	movwf var1_spg		; var1_spg = "0000XXXX", X = 1 contact ferm
	swapf var1_spg,w		; var1_spg = "XXXX0000", X = 1 contact ferm
	iorwf mat_in0,f		; 'OU' mat_in0 = OU ("XXXXYYYY", X = 1 contact ferm, mat_in0)

	bsf   PORTB,CLKMAT
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	movlw B'00001111'
	andwf mat_in1,f
	comf  PORTC,w
	bcf   PORTB,CLKMAT
	andlw B'00001011'
	btfss PORTB,0
	iorlw B'00000100'
	iorwf mat_in1,f
	movwf var1_spg
	swapf var1_spg,w
	iorwf mat_in1,f

	bsf   PORTB,CLKMAT
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	movlw B'00001111'
	andwf mat_in2,f
	comf  PORTC,w
	bcf   PORTB,CLKMAT
	andlw B'00001011'
	btfss PORTB,0
	iorlw B'00000100'
	iorwf mat_in2,f
	movwf var1_spg
	swapf var1_spg,w
	iorwf mat_in2,f

	bsf   PORTB,CLKMAT
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	movlw B'00001111'
	andwf mat_in3,f
	comf  PORTC,w
	bcf   PORTB,CLKMAT
	andlw B'00001011'
	btfss PORTB,0
	iorlw B'00000100'
	iorwf mat_in3,f
	movwf var1_spg
	swapf var1_spg,w
	iorwf mat_in3,f

	bsf   PORTB,CLKMAT
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	movlw B'00001111'
	andwf mat_in4,f
	comf  PORTC,w
	bcf   PORTB,CLKMAT
	andlw B'00001011'
	btfss PORTB,0
	iorlw B'00000100'
	iorwf mat_in4,f
	movwf var1_spg
	swapf var1_spg,w
	iorwf mat_in4,f

	bsf   PORTB,CLKMAT
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	movlw B'00001111'
	andwf mat_in5,f
	comf  PORTC,w
	bcf   PORTB,CLKMAT
	andlw B'00001011'
	btfss PORTB,0
	iorlw B'00000100'
	iorwf mat_in5,f
	movwf var1_spg
	swapf var1_spg,w
	iorwf mat_in5,f

	bsf   PORTB,CLKMAT
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	movlw B'00001111'
	andwf mat_in6,f
	comf  PORTC,w
	bcf   PORTB,CLKMAT
	andlw B'00001011'
	btfss PORTB,0
	iorlw B'00000100'
	iorwf mat_in6,f
	movwf var1_spg
	swapf var1_spg,w
	iorwf mat_in6,f

	bsf   PORTB,CLKMAT
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	movlw B'00001111'
	andwf mat_in7,f
	comf  PORTC,w
;;;;	bcf   PORTB,CLKMAT	; report plus loin, car la sortie /QS
	andlw B'00001011'		; est spciale.
	btfss PORTB,0
	iorlw B'00000100'
	bcf   PORTB,CLKMAT	; report ici
	iorwf mat_in7,f
	movwf var1_spg
	swapf var1_spg,w
	iorwf mat_in7,f

	bsf   PORTB,CLKMAT	; Utilise la sortie /QS
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	movlw B'00001111'
	andwf mat_in8,f
	comf  PORTC,w
;;;	bcf   PORTB,CLKMAT	; Surtout pas sur /QS
	andlw B'00001011'
	btfss PORTB,0
	iorlw B'00000100'
	iorwf mat_in8,f
	movwf var1_spg
	swapf var1_spg,w
	iorwf mat_in8,f

	bcf   PORTB,CLKMAT	; RAZ
	return


;-----------------------------------------------------------------------------
; SSPG_CPT7 : Commande du crneau pour les servos. Modif 15/02/2020
; Variante pour des servo autres que 1,5 - 2 - 2,5 msec pour un dbattement de 180.
; 
; Utilise le 'Timer1' en mode 'Compare' sans interruption.
; si TMR1 = CCP1 alors la sortie PORTC(2) passe  1 (modif du CI obligatoire).
; Le signal en sortie de CD4028 est invers par rapport au crneau en sortie du PIC
; Commande de 5 servos maximum, car 5 * 4 msec = 20 msec max par servo
;
; Si cpt_srv(0)=1, il faut positionner la sortie (crneau de 1  2 msec)
; Avec le prdiviseur  8, le pas du Timer1 = 1,6 s  20 Mhz
; Position du servo  min,  moyen,  max :
;   srv_x   =    0       |  128       |   255
;   Crneau =  1,0 msec  |  1,5 msec  |   2,0 msec
;   Timer1  =  625 min   |  937 moyen |  1250 max
;
;   srv_x   =    0        |   128       |   255
;
; Servo type 1 => Formule : 625 + 2,5*srv_x
;   Crneau =  1,00 msec  |  1,50 msec  |   2,00 msec
;
; Servo type 2 => Formule : 554 + 3*srv_x
;   Crneau =  0,88 msec  |  1,50 msec  |   2,11 msec
;
; Servo type 3 => Formule : 490 + 3,5*srv_x
;   Crneau =  0,78 msec  |  1,50 msec  |   2,21 msec
;
; Servo type 4 => Formule : 426 + 4*srv_x
;   Crneau =  0,68 msec  |  1,50 msec  |   2,31 msec
;
; Servo type 5 => Formule : 361 + 4,5*srv_x
;   Crneau =  0,57 msec  |  1,50 msec  |   2,41 msec
;
cal_servo
	btfsc cpt_srv,0		; cpt_srv = nde servo en cours(7bits) + 1bit (1=activer timer1)
	return				; return si cpt_srv(0) = 1

	movf	cpt_srv,w		; cpt_srv = 2 * numro du servo en cours.
	movwf	var1_spg
	bcf	STATUS,C		; 0 - > CARRY (dcalage de 2 bits vers la droite = division par 2)
	rrf	var1_spg,w
	addlw	srv_0
	movwf	FSR			; FSR = srv_0  srv_5.
	movf	INDF,w		; w = [srv_0]  [srv_5].
	BANK1
;
; Converti la valeur 0-255 en xxx*srv_x + yyy.
;   val_srv ===> val_srvH/val_srvL
;
	movwf	val_srv		; val_srv = valeur de consigne entre 0 et 255 = valeur de servo[X].
	movwf	val_srvL
	clrf	val_srvH
	bcf		STATUS,C		; 0 - > CARRY (avant le dcalage de 1 bit vers la gauche = multiplication par 2).

; Suivant le type de servo dsir, il faut modifier le code ci-dessous,
; et enlever les commentaires que pour le type de servo dsir.	

; Servo type 1 => Formule : 625 + 2,5*srv_x
;   Crneau =  1,00 msec  |  1,50 msec  |   2,00 msec
if type_servo == 1 
	rlf		val_srvL,f
	rlf		val_srvH,f		; val_srvL/H = valeur * 2 
	bcf		STATUS,C		; 0 - > CARRY (dcalage de 1 bit vers la droite = division par 2)
	rrf		val_srv,f		; val_srv = valeur / 2
	movf	val_srv,w
	addwf 	val_srvL,f
	btfsc	STATUS,C
	incf	val_srvH,f		; val_srvH/L = valeur * 2,5
	movlw	.113
	addwf	val_srvL,f		; +625 = 2*256 + 113
	movlw	.2
	btfsc	STATUS,C
	movlw	.2+.1
	addwf	val_srvH,f		; 625 + 2,5*srv_x
endif

; Servo type 2 => Formule : 554 + 3*srv_x
;   Crneau =  0,88 msec  |  1,50 msec  |   2,11 msec
if type_servo == 2
	rlf		val_srvL,f
	rlf		val_srvH,f		; val_srvL/H = valeur * 2 
	movf	val_srv,w
	addwf 	val_srvL,f
	btfsc	STATUS,C
	incf	val_srvH,f		; val_srvH/L = valeur * 3
	movlw	.42				; +554 = 2*256 + 42
	addwf	val_srvL,f
	movlw	.2
	btfsc	STATUS,C
	movlw	.2+.1
	addwf	val_srvH,f		; val_srvH/L = 554 + 3*srv_x
endif

; Servo type 3 => Formule : 490 + 3,5*srv_x
;   Crneau =  0,78 msec  |  1,50 msec  |   2,21 msec
if type_servo == 3
	rlf		val_srvL,f
	rlf		val_srvH,f		; val_srvL/H = valeur * 2 
	movf	val_srv,w
	addwf 	val_srvL,f
	btfsc	STATUS,C
	incf	val_srvH,f		; val_srvH/L = valeur * 3 
	bcf		STATUS,C		; 0 - > CARRY (dcalage de 1 bit vers la droite = division par 2)
	rrf		val_srv,f		; val_srv = valeur / 2
	movf	val_srv,w
	addwf 	val_srvL,f
	btfsc	STATUS,C
	incf	val_srvH,f		; val_srvH/L = valeur * 3,5
	movlw	.234			; +490 = 1*256 + 234
	addwf	val_srvL,f
	movlw	.1
	btfsc	STATUS,C
	movlw	.1+.1
	addwf	val_srvH,f		; val_srvH/L = 490 + 3,5*srv_x
endif

; Servo type 4 => Formule : 426 + 4*srv_x
;   Crneau =  0,68 msec  |  1,50 msec  |   2,31 msec
if type_servo == 4
	rlf		val_srvL,f
	rlf		val_srvH,f		; val_srvL/H = valeur * 2 
	bcf		STATUS,C		; 0 - > CARRY (avant le dcalage de 1 bit vers la gauche = multiplication par 2).
	rlf		val_srvL,f
	rlf		val_srvH,f		; val_srvL/H = valeur * 4
	movlw	.170
	addwf	val_srvL,f		; +426 = 1*256 + 170
	movlw	.1
	btfsc	STATUS,C
	movlw	.1+.1
	addwf	val_srvH,f		; val_srvH/L = 426 + 4*srv_x
endif
;
; Servo type 5 => Formule : 361 + 4,5*srv_x
;   Crneau =  0,57 msec  |  1,50 msec  |   2,41 msec
if type_servo == 5
	rlf		val_srvL,f
	rlf		val_srvH,f		; val_srvL/H = valeur * 2 
	bcf		STATUS,C		; 0 - > CARRY (avant le dcalage de 1 bit vers la gauche = multiplication par 2).
	rlf		val_srvL,f
	rlf		val_srvH,f		; val_srvL/H = valeur * 4
	bcf		STATUS,C		; 0 - > CARRY (dcalage de 1 bit vers la droite = division par 2)
	rrf		val_srv,f		; val_srv = valeur / 2
	movf	val_srv,w
	addwf 	val_srvL,f
	btfsc	STATUS,C
	incf	val_srvH,f		; val_srvH/L = valeur * 4,5
	movlw	.105
	addwf	val_srvL,f		; +361 = 1*256 + 105
	movlw	.1
	btfsc	STATUS,C
	movlw	.1+.1
	addwf	val_srvH,f		; val_srvH/L = 361 + 4,5*srv_x
endif
;
; Mise  1 de la sortie PORTC(2)  (forage par comparaison, 0=0)
;
	BANK0
	bcf	T1CON,TMR1ON	; Arrte le timer 'TMR1' + raz
	bcf	PIR1,TMR1IF
	clrf	TMR1L
	clrf	TMR1H
	clrf	CCPR1L
	clrf	CCPR1H
	movlw	B'00001000'		; Force la sortie  1 si galit TMR1 = CCPR1
	movwf	CCP1CON
	bsf	T1CON,TMR1ON	; Lance le timer 'TMR1'
;
; Dpart du crneau qui retombera tout seul.
;
	BANK1
	movf	val_srvH,w
	BANK0
	movwf	CCPR1H
	BANK1
	movf	val_srvL,w
	BANK0
	movwf	CCPR1L
	movlw	B'00001001'		; La sortie passera  0 quand le compteur TMR1 = CCPR1
	movwf	CCP1CON
	return


;-----------------------------------------------------------------------------
; SSPG_CPT0 : Envoi uniquement les 8 bits des sorties systme du PIC (servos,,,)
;             Lecture simultane des entres [2].
;
; A 20 Mhz, une instruction = 200 ns, il ne faut pas faire deux
; critures/lecture successives sur le mme port, il faut insrer un nop.
; Servos : On envoi les bits : cpt_srv(3), cpt_srv(2) et cpt_srv(1)
; + bits 'cpt' en attendant une autre utilisation
;
maj_out6
	clrf	in_in1		; Effacement de l'octet de stockage des entres [2]

	btfss PORTC,DATIN2
	bsf	in_in1,6
	btfss	cpt,7			; 7 : 'cpt' en attendant une autre utilisation
	CK_DOWN_OUT
	btfsc	cpt,7			; 7 : 'cpt' en attendant une autre utilisation
	CK_UP_OUT

	btfss PORTC,DATIN2
	bsf	in_in1,4
	btfss	cpt,6			; 6 : 'cpt' en attendant une autre utilisation
	CK_DOWN_OUT
	btfsc	cpt,6			; 6 : 'cpt' en attendant une autre utilisation
	CK_UP_OUT

	btfss PORTC,DATIN2
	bsf	in_in1,2
	btfss	cpt_srv,2		; 5 : n servo 0X00
	CK_DOWN_OUT
	btfsc	cpt_srv,2		; 5 : n servo 0X00
	CK_UP_OUT

	btfss PORTC,DATIN2
	bsf	in_in1,0
	btfss	cpt_srv,3		; 4 : n servo X000
	CK_DOWN_OUT
	btfsc	cpt_srv,3		; 4 : n servo X000
	CK_UP_OUT

	btfss PORTC,DATIN2
	bsf	in_in1,1
	btfss	cpt,5			; 3 : 'cpt' en attendant une autre utilisation
	CK_DOWN_OUT
	btfsc cpt,5			; 3 : 
	CK_UP_OUT

	btfss PORTC,DATIN2
	bsf	in_in1,3		; 2 :
	btfss	cpt,4
	CK_DOWN_OUT
	btfsc cpt,4			; 2 :
	CK_UP_OUT

	btfss PORTC,DATIN2
	bsf	in_in1,5
	btfss	cpt,3			; 1 :
	CK_DOWN_OUT
	btfsc cpt,3			; 1 :
	CK_UP_OUT

	btfss PORTC,DATIN2
	bsf	in_in1,7
	btfss	cpt_srv,1		; 0 : n servo 00X0
	CK_DOWN_OUT
	btfsc	cpt_srv,1		; 0 : n servo 00X0
	CK_UP_OUT
	return


;-----------------------------------------------------------------------------
; SSPG_CPT1 : Envoi les donnes vers les relais et les leds multiplexes
;             Envoi des bits des sorties  16  23  (IC16-ULN2008)
;
maj_out5
	clrf	in_in2		; Effacement de l'octet de stockage des entres [2]

	btfss PORTC,DATIN2
	bsf	in_in2,6
	btfss out_out5,7
	bcf   PORTB,DATOUT
	btfss out_out25,7		; (out 47)
	CK_DOWN_OUT
	btfsc out_out5,7
	bsf   PORTB,DATOUT
	btfsc out_out25,7
	CK_UP_OUT

	btfss PORTC,DATIN2
	bsf	in_in2,4
	btfss out_out5,6
	bcf   PORTB,DATOUT
	btfss out_out25,6		; (out 46)
	CK_DOWN_OUT
	btfsc out_out5,6
	bsf   PORTB,DATOUT
	btfsc out_out25,6
	CK_UP_OUT

	btfss PORTC,DATIN2
	bsf	in_in2,2
	btfss out_out5,5
	bcf   PORTB,DATOUT
	btfss out_out25,5		; (out 45)
	CK_DOWN_OUT
	btfsc out_out5,5
	bsf   PORTB,DATOUT
	btfsc out_out25,5
	CK_UP_OUT

	btfss PORTC,DATIN2
	bsf	in_in2,0
	btfss out_out5,4
	bcf   PORTB,DATOUT
	btfss	out_out25,4
	CK_DOWN_OUT
	btfsc out_out5,4
	bsf   PORTB,DATOUT
	btfsc out_out25,4
	CK_UP_OUT

	btfss PORTC,DATIN2
	bsf	in_in2,1
	btfss out_out5,3
	bcf   PORTB,DATOUT
	btfss	out_out25,3
	CK_DOWN_OUT
	btfsc out_out5,3
	bsf   PORTB,DATOUT
	btfsc out_out25,3
	CK_UP_OUT

	btfss PORTC,DATIN2
	bsf	in_in2,3
	btfss out_out5,2
	bcf   PORTB,DATOUT
	btfss	out_out25,2
	CK_DOWN_OUT
	btfsc out_out5,2
	bsf   PORTB,DATOUT
	btfsc out_out25,2
	CK_UP_OUT

	btfss PORTC,DATIN2
	bsf	in_in2,5
	btfss out_out5,1
	bcf   PORTB,DATOUT
	btfss	out_out25,1
	CK_DOWN_OUT
	btfsc out_out5,1
	bsf   PORTB,DATOUT
	btfsc out_out25,1
	CK_UP_OUT

	btfss PORTC,DATIN2
	bsf	in_in2,7
	btfss out_out5,0
	bcf   PORTB,DATOUT
	btfss	out_out25,0
	CK_DOWN_OUT
	btfsc out_out5,0
	bsf   PORTB,DATOUT
	btfsc out_out25,0
	CK_UP_OUT
	return


;-----------------------------------------------------------------------------
; SSPG_CPT2 : Envoi les donnes vers les relais et les leds multiplexes
;             Envoi des bits des sorties  8  15  (IC15-ULN2008)
maj_out4
	clrf	in_in3		; Effacement de l'octet de stockage des entres [2]

	btfss PORTC,DATIN2
	bsf	in_in3,6
	btfss out_out4,7
	bcf   PORTB,DATOUT
	btfss	out_out24,7
	CK_DOWN_OUT
	btfsc out_out4,7
	bsf   PORTB,DATOUT
	btfsc out_out24,7
	CK_UP_OUT

	btfss PORTC,DATIN2
	bsf	in_in3,4
	btfss out_out4,6
	bcf   PORTB,DATOUT
	btfss	out_out24,6
	CK_DOWN_OUT
	btfsc out_out4,6
	bsf   PORTB,DATOUT
	btfsc out_out24,6
	CK_UP_OUT

	btfss PORTC,DATIN2
	bsf	in_in3,2
	btfss out_out4,5
	bcf   PORTB,DATOUT
	btfss	out_out24,5
	CK_DOWN_OUT
	btfsc out_out4,5
	bsf   PORTB,DATOUT
	btfsc out_out24,5
	CK_UP_OUT

	btfss PORTC,DATIN2
	bsf	in_in3,0
	btfss out_out4,4
	bcf   PORTB,DATOUT
	btfss	out_out24,4
	CK_DOWN_OUT
	btfsc out_out4,4
	bsf   PORTB,DATOUT
	btfsc out_out24,4
	CK_UP_OUT

	btfss PORTC,DATIN2
	bsf	in_in3,1
	btfss out_out4,3
	bcf   PORTB,DATOUT
	btfss	out_out24,3
	CK_DOWN_OUT
	btfsc out_out4,3
	bsf   PORTB,DATOUT
	btfsc out_out24,3
	CK_UP_OUT

	btfss PORTC,DATIN2
	bsf	in_in3,3
	btfss out_out4,2
	bcf   PORTB,DATOUT
	btfss	out_out24,2
	CK_DOWN_OUT
	btfsc out_out4,2
	bsf   PORTB,DATOUT
	btfsc out_out24,2
	CK_UP_OUT

	btfss PORTC,DATIN2
	bsf	in_in3,5
	btfss out_out4,1
	bcf   PORTB,DATOUT
	btfss	out_out24,1
	CK_DOWN_OUT
	btfsc out_out4,1
	bsf   PORTB,DATOUT
	btfsc out_out24,1
	CK_UP_OUT

	btfss PORTC,DATIN2
	bsf	in_in3,7
	btfss out_out4,0
	bcf   PORTB,DATOUT
	btfss	out_out24,0
	CK_DOWN_OUT
	btfsc out_out4,0
	bsf   PORTB,DATOUT
	btfsc out_out24,0
	CK_UP_OUT
	return

;-----------------------------------------------------------------------------
; SSPG_CPT3 : Envoi les donnes vers les relais et les leds multiplexes
;             Envoi des bits des sorties  1  7  (IC14-ULN2008)
maj_out3
	clrf	in_in4		; Effacement de l'octet de stockage des entres [2]

	btfss PORTC,DATIN2
	bsf	in_in4,6
	btfss out_out3,7
	bcf   PORTB,DATOUT
	btfss	out_out23,7
	CK_DOWN_OUT
	btfsc out_out3,7
	bsf   PORTB,DATOUT
	btfsc out_out23,7
	CK_UP_OUT

	btfss PORTC,DATIN2
	bsf	in_in4,4
	btfss out_out3,6
	bcf   PORTB,DATOUT
	btfss	out_out23,6
	CK_DOWN_OUT
	btfsc out_out3,6
	bsf   PORTB,DATOUT
	btfsc out_out23,6
	CK_UP_OUT

	btfss PORTC,DATIN2
	bsf	in_in4,2
	btfss out_out3,5
	bcf   PORTB,DATOUT
	btfss	out_out23,5
	CK_DOWN_OUT
	btfsc out_out3,5
	bsf   PORTB,DATOUT
	btfsc out_out23,5
	CK_UP_OUT

	btfss PORTC,DATIN2
	bsf	in_in4,0
	btfss out_out3,4
	bcf   PORTB,DATOUT
	btfss	out_out23,4
	CK_DOWN_OUT
	btfsc out_out3,4
	bsf   PORTB,DATOUT
	btfsc out_out23,4
	CK_UP_OUT

	btfss PORTC,DATIN2
	bsf	in_in4,1
	btfss out_out3,3
	bcf   PORTB,DATOUT
	btfss	out_out23,3
	CK_DOWN_OUT
	btfsc out_out3,3
	bsf   PORTB,DATOUT
	btfsc out_out23,3
	CK_UP_OUT

	btfss PORTC,DATIN2
	bsf	in_in4,3
	btfss out_out3,2
	bcf   PORTB,DATOUT
	btfss	out_out23,2
	CK_DOWN_OUT
	btfsc out_out3,2
	bsf   PORTB,DATOUT
	btfsc out_out23,2
	CK_UP_OUT

	btfss PORTC,DATIN2
	bsf	in_in4,5
	btfss out_out3,1
	bcf   PORTB,DATOUT
	btfss	out_out23,1
	CK_DOWN_OUT
	btfsc out_out3,1
	bsf   PORTB,DATOUT
	btfsc out_out23,1

	CK_UP_OUT
	btfss PORTC,DATIN2
	bsf	in_in4,7
	btfss out_out3,0
	bcf   PORTB,DATOUT
	btfss	out_out23,0
	CK_DOWN_OUT
	btfsc out_out3,0
	bsf   PORTB,DATOUT
	btfsc out_out23,0
	CK_UP_OUT
	return

;-----------------------------------------------------------------------------
; SSPG_CPT4 : Envoi les donnes vers les relais et les leds multiplexes
;             Envoi des bits des leds  8  11 et A  D (IC13-ULN2008)
maj_out2
	clrf	in_in5		; Effacement de l'octet de stockage des entres [2]

	btfss PORTC,DATIN2
	bsf	in_in5,6
	btfss out_out2,7
	bcf   PORTB,DATOUT
	btfss	out_out22,7
	CK_DOWN_OUT
	btfsc out_out2,7
	bsf   PORTB,DATOUT
	btfsc out_out22,7
	CK_UP_OUT

	btfss PORTC,DATIN2
	bsf	in_in5,4
	btfss out_out2,6
	bcf   PORTB,DATOUT
	btfss	out_out22,6
	CK_DOWN_OUT
	btfsc out_out2,6
	bsf   PORTB,DATOUT
	btfsc out_out22,6
	CK_UP_OUT

	btfss PORTC,DATIN2
	bsf	in_in5,2
	btfss out_out2,5
	bcf   PORTB,DATOUT
	btfss	out_out22,5
	CK_DOWN_OUT
	btfsc out_out2,5
	bsf   PORTB,DATOUT
	btfsc out_out22,5
	CK_UP_OUT

	btfss PORTC,DATIN2
	bsf	in_in5,0
	btfss out_out2,4
	bcf   PORTB,DATOUT
	btfss	out_out22,4
	CK_DOWN_OUT
	btfsc out_out2,4
	bsf   PORTB,DATOUT
	btfsc out_out22,4
	CK_UP_OUT

	btfss PORTC,DATIN2
	bsf	in_in5,1
	btfss out_out2,3
	bcf   PORTB,DATOUT
	btfss	out_out22,3
	CK_DOWN_OUT
	btfsc out_out2,3
	bsf   PORTB,DATOUT
	btfsc out_out22,3
	CK_UP_OUT

	btfss PORTC,DATIN2
	bsf	in_in5,3
	btfss out_out2,2
	bcf   PORTB,DATOUT
	btfss	out_out22,2
	CK_DOWN_OUT
	btfsc out_out2,2
	bsf   PORTB,DATOUT
	btfsc out_out22,2
	CK_UP_OUT

	btfss PORTC,DATIN2
	bsf	in_in5,5
	btfss out_out2,1
	bcf   PORTB,DATOUT
	btfss	out_out22,1
	CK_DOWN_OUT
	btfsc out_out2,1
	bsf   PORTB,DATOUT
	btfsc out_out22,1
	CK_UP_OUT

	btfss PORTC,DATIN2
	bsf	in_in5,7
	btfss out_out2,0
	bcf   PORTB,DATOUT
	btfss	out_out22,0
	CK_DOWN_OUT
	btfsc out_out2,0
	bsf   PORTB,DATOUT
	btfsc out_out22,0
	CK_UP_OUT

	return

;-----------------------------------------------------------------------------
; SSPG_CPT5 : Envoi les donnes vers les relais et les leds multiplexes
;             Envoi des bits des leds  1  7 et validation des 74HCT4094
maj_out1
	btfss out_out1,7
	bcf   PORTB,DATOUT
	btfss	out_out21,7
	CK_DOWN_OUT
	btfsc out_out1,7
	bsf   PORTB,DATOUT
	btfsc out_out21,7
	CK_UP_OUT

	btfss out_out1,6
	bcf   PORTB,DATOUT
	btfss	out_out21,6
	CK_DOWN_OUT
	btfsc out_out1,6
	bsf   PORTB,DATOUT
	btfsc out_out21,6
	CK_UP_OUT

	btfss out_out1,5
	bcf   PORTB,DATOUT
	btfss	out_out21,5
	CK_DOWN_OUT
	btfsc out_out1,5
	bsf   PORTB,DATOUT
	btfsc out_out21,5
	CK_UP_OUT

	btfss out_out1,4
	bcf   PORTB,DATOUT
	btfss	out_out21,4
	CK_DOWN_OUT
	btfsc out_out1,4
	bsf   PORTB,DATOUT
	btfsc out_out21,4
	CK_UP_OUT

	btfss out_out1,3
	bcf   PORTB,DATOUT
	btfss	out_out21,3
	CK_DOWN_OUT
	btfsc out_out1,3
	bsf   PORTB,DATOUT
	btfsc out_out21,3
	CK_UP_OUT

	btfss out_out1,2
	bcf   PORTB,DATOUT
	btfss	out_out21,2
	CK_DOWN_OUT
	btfsc out_out1,2
	bsf   PORTB,DATOUT
	btfsc out_out21,2
	CK_UP_OUT

	btfss out_out1,1
	bcf   PORTB,DATOUT
	btfss	out_out21,1
	CK_DOWN_OUT
	btfsc out_out1,1
	bsf   PORTB,DATOUT
	btfsc out_out21,1
	CK_UP_OUT

	btfss out_out1,0
	bcf   PORTB,DATOUT
	btfss	out_out21,0
	CK_DOWN_OUT
	btfsc out_out1,0
	bsf   PORTB,DATOUT
	btfsc out_out21,0
	CK_UP_OUT

	nop
	nop
	bsf   PORTB,LODOUT
	nop
	nop
	nop
	bcf   PORTB,LODOUT	; Impulsion de validation de 800 ns
	return



;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------

; Dbut du code d'initialisation en page n1, 0x800 pour laisser de la place au programme
; principal en page n0 0x000-0x7FF.
; Attention, ici les 'call' ne font appel qu' des sous-programmes en page n1
;
	ORG     0x800		; Le code dbute  l'origine de la page n1
init	call    init_port		; Initialise les ports A, B et C et les variables d'entres/sortie
	call    init_can		; Initialise les ports CAN (conversion analogique numrique)
	call    init_pwm		; Initialise toutes les variables de la partie pwm
	call    init_led		; Initialise toutes les variables de la partie led
	call    init_srv		; Initialise CCPxCON et les variables de la partie servo
	call    init_rs232	; Initialise la liaison srie
	call    init_timer	; Initialise le Timer0 (TMR0)
	call    init_det		; Initialise les variables de dtection d'occupation des cantons
	call    init_it		; Initialise la gestion des interruptions
	return			; Retourne en page n 0 (0x000 - 0x7FF)

;-----------------------------------------------------------------------------
; SSPG_INIT : Initialise les ports A, B et C et les variables d'entres / sortie
;
init_port
	clrf    PORTB		; initialisation de toutes les sorties  0
	clrf    PORTA
	clrf    PORTC

	BANK1				; passer en banque1
	movlw   B'00101111'	; PORTA bit4 en sortie (B4 = led clignotante)
	movwf   TRISA

	movlw   B'00001001'	; PORTB en sortie(sauf en entre B3 = capteur de canton, B0 = in mux)
	movwf   TRISB

	movlw   B'10011011'	; PORTC en entre, sauf C2 = Servo, C6 = rx max232 et C5 = OUT en V3.0
	movwf   TRISC
	BANK0				; passer en banque0

	clrf    out_out1		; out_out(x) = 0
	clrf    out_out2
	clrf    out_out3
	clrf    out_out4
	clrf    out_out5

	clrf    out_out21		; nouveau en V3.0
	clrf    out_out22
	clrf    out_out23
	clrf    out_out24
	clrf    out_out25

	clrf    out_pwm1		; out_pwm(x) = 0
	clrf    out_pwm2
	clrf    out_pwm3
	clrf    out_pwm4

	clrf    mat_in0		; mat_in(x) = 0
	clrf    mat_in1
	clrf    mat_in2
	clrf    mat_in3
	clrf    mat_in4
	clrf    mat_in5
	clrf    mat_in6
	clrf    mat_in7
	clrf    mat_in8

	clrf    in_in1
	clrf    in_in2
	clrf    in_in3
	clrf    in_in4
	clrf    in_in5
	return

;-----------------------------------------------------------------------------
; SSPG_INIT : Initialise le convertisseur analogique / numrique
;
; ADCON0
;   ADCS10  =   10 = F conversion = Fosc/32 car le quartz est  20 Mhz
;   CHS210  =  xxx = n du canal de conversion CAN en cours
;   GO/DONE =    0 = A mettre  '1' aprs une conversion
;   nc      =    0
;   ADON    =    1 = Mise en route du convertisseur
;
; ADCON1
;   ADFM    =    0 = Rsultat justifi  gauche
;   ADCS2   =    0 = F conversion = Fosc/32 car le quartz est  20 Mhz
;   nc nc   =   00
;   PCFG    = 0010 = RA0  RA4 = entres analogiques, Vref = +5 V et +0 V,
;
init_can
	BANK0				; passer en banque0
	movlw   B'10000001'	; Fosc/32 + Convertisseur = ON
	movwf   ADCON0
	BANK1				; passer en banque1
	movlw   B'00000010'	; Rsultat de conversion dans ADRESH (8bits) + ADRESL (4bits)
	movwf   ADCON1		; RESADRH 8bits toutes entres can en mode can
	BANK0				; passer en banque0
	movlw   .128		; 128 = position mdiane des potentiomtres.
	movwf   can_r0
	movwf   can_r1
	movwf   can_r2
	movwf   can_r3
	movwf   can_r4
	return

;-----------------------------------------------------------------------------
; SSPG_INIT : Initialise les variables de la partie PWM (consignes voies)
;
init_pwm
	clrf    pwm_cpt		; pwm_cpt  = 0
	clrf    pwm_r0		; pwm_r(x) = 0 = vitesse par canton
	clrf    pwm_r1		; Rails pas aliment au dpart
	clrf    pwm_r2
	clrf    pwm_r3
	clrf    pwm_r4
	clrf    pwm_r5
	clrf    pwm_r6
	clrf    pwm_r7
	clrf    pwm_r8
	clrf    pwm_r9
	clrf    pwm_r10
	clrf    pwm_r11
	clrf    pwm_r12
	clrf    pwm_r13
	clrf    pwm_r14
	clrf    pwm_r15
	clrf    pwm_s0_7		; pwm_s(x) = 0 = sens de dplacement par canton
	clrf    pwm_s8_15
	return

;-----------------------------------------------------------------------------
; SSPG_INIT : Initialise les variables de la partie leds multiplxes
;
init_led
	clrf  led_on0	; led_on(x) = 0
	clrf  led_on1	; Etient les leds
	clrf  led_on01
	clrf  led_on2
	clrf  led_on3
	clrf  led_on23

	movlw 0xff
	movwf led_cl0	; led_c(x) = 0xFF
	movwf led_cl1	; Pas de clignotement
	movwf led_cl01
	movwf led_cl2
	movwf led_cl3
	movwf led_cl23

	clrf  led_in0	; led_in(x) = 0
	clrf  led_in1	; Clignotement pas altrnes
	clrf  led_in01
	clrf  led_in2
	clrf  led_in3
	clrf  led_in23
	return

;-----------------------------------------------------------------------------
; SSPG_INIT : Initialise la configuration CCP1CON et les variables de la partie servo
;
; T1CON = B'00110001'
;   nc  nc  = 00 = Pas utilis
;   T1CKPS  = 11 = Prdiviseur du Timer1 = 8,      (un pas = 1,6 sec  20 Mhz)
;   T1OSCEN =  0 = Oscillateur interne  off
;   T1SYN   =  0 = Pas utile si TMR1ON = 1
;   TMR1CS  =  0 = Source prdiviseur = Fosc/4/T1CKPS
;   TMR1ON  =  1 = Autorise le timer  fonctionner (le compteur dfile)
;
; CCP1CON = B'00001001'
;   nc  nc  = 00 = pas utilis
;   CCP1X   =  0 = pas utilis, module compare, bit x
;   CCP1Y   =  0 = pas utilis, module compare, bit y
;   CCP1M   = 1000 = module compare => si galit place la sortie  1 (+ bit CCP1IF =1)
;
;   Crneau =  1,0 msec  |  1,5 msec  |   2,0 msec
;   Timer1  =  625 min   |  937 moyen |  1250 max
;
init_srv
	movlw   B'00110001'	; Pas du Timer1 = 1,6 s  20 Mhz
	movwf   T1CON
	movlw   B'00001001'
	movwf   CCP1CON		; Mode 'compare' place la sortie  0 sur galit + bit CCP1IF =1
	movlw   .128		; 128 = position centrale pour les servos = 1,5 msec.
	movwf   srv_0		; 5 servos
	movwf   srv_1
	movwf   srv_2
	movwf   srv_3
	movwf   srv_4
	clrf    cpt_srv		; raz variable n de servo
	
; 3 lignes uniquement pour dbug servo_0 = 0,   servo_1 = 128,    servo_2 = 255
;;; clrf	srv_0
;;;	movlw   .255
;;;	movwf   srv_2
	
	return

;-----------------------------------------------------------------------------
; SSPG_INIT : Initialise la liaison srie
;
; Calcul de SPBRG = (Fosc / ( 16 * Bd[liaison srie])) - 1
; Vitesse baud =  9600   19200  [38400]
; Frq = 20 Mhz    129     64     [32]
;
; TXSTA = B'00100100'
;   CSRC = 0 = pas utilis
;   TX9  = 0 = caractre sur 8 bits
;   TXEN = 1 = transmission autorise
;   SYNC = 0 = mode asynchrone
;   nc   = 0
;   BRGH = 1 = mode haute vitesse (Fosc / 16)
;   TRMT = 0 = registre d'envoi vide
;   TX9D = 0 = valeur du 9 eme bit
;
; RCSTA = B'10010000'
;   SPEN  = 1 = port srie autoris
;   RX9   = 0 = caractres sur 8 bits
;   SREN  = 0 pas utilis
;   CREN  = 1 = autorise la rception en continue
;   ADDEN = 0 pas utilis
;   FERR  = 0 = Pas d'erreur de trame
;   OERR  = 0 pas utilis
;   RX9D  = 0 = 9m bit ou bit de parit
;
init_rs232
	BANK1				; passer en banque1
	movlw   .32			; 32 = 38400 bauds (hight vitesse)
	movwf   SPBRG		; SPBRG = Serial Port Bit Rate Generator = (Fosc/(16*D))-1
					; (20.000.000/(16*38.400))-1 = 32
	movlw   B'00100100'	; asynchrone, tx enable, hight vit
	movwf   TXSTA
	BANK0				; passer en banque0
	movlw   B'10010000'
	movwf   RCSTA		; port srie autoris
	bcf     PIR1,RCIF		; Le flag "buffer de rception" est vid
	clrf    indicateurs	; Carte dslectionne
	clrf    rs_adr		; 
	clrf    rs_mode		; rs_mode = 0 = attente d'un caractre de commande
	return

;-----------------------------------------------------------------------------
; SSPG_INIT : Initialise le timer principal = Timer0 (TMR0)
; pour avoir une interrution toutes les 250 s.
;
; OPTION_REG = B'11000010'
;   RBPU   =   1 = Pas de rsistance de rappel sur le port B
;   INTEDG =   1 = Interruption dur front montant RB0 (pas utilis)
;   T0CS   =   0 = Source interne pour TMR0
;   TOSE   =   0 = Pas utilis
;   PSA    =   0 = Prdiviseur pour le Timer0
;   PS     = 011 = Prdiviseur pour Timer0 = 1/16 => (4*16)/20 Mhz = 3,2 sec
;
init_timer
	BANK1				; passer en banque1
	movlw   B'11000011'	; base de temps de TMR0 = 3,2 sec  20 Mhz
	movwf   OPTION_REG
	BANK0				; passer en banque0
	return

;-----------------------------------------------------------------------------
; SSPG_INIT : Initialise les variables de dtection d'occupation des cantons
;
init_det
	clrf    det_i1
	clrf    det_i2
	clrf    det_i1m
	clrf    det_i2m
	BANK1				; passer en banque1
	clrf    det_i1_1		; Partie des variables en BANK1
	clrf    det_i1_2
	clrf    det_i1_3
	clrf    det_i1_4
	clrf    det_i1_5
	clrf    det_i1_6
	clrf    det_i2_1
	clrf    det_i2_2
	clrf    det_i2_3
	clrf    det_i2_4
	clrf    det_i2_5
	clrf    det_i2_6
	BANK0				; passer en banque0
	return

;-----------------------------------------------------------------------------
; SSPG_INIT : Initialise la gestion des interruptions
;
; INTCON = B'10100000'
;   GIE    = 1 = Interruptions globalement autorises
;   PEIE   = 0 = Pas d'interruption de priphrique
;   T0IE   = 1 = Interruption autorises du timer TRM0
;   INTE   = 0 = Pas d'interruption du port PORTB
;   RBIE   = 0 = Pas d'interruption du port PORTB
;   T0IF   = 0 = Bit d'tat (dbordement TRM0)
;   INTF   = 0 = Bit d'tat (RB0 a boug)
;   RBIF   = 0 = Bit d'tat (RB4-7 a boug)
;
; PIE1  = B'00000000'
;   PSPIE  = 0 = Pas d'interruption du port //
;   ADIE   = 0 = Pas d'interruption conv A/D
;   RCIE   = 0 = Pas d'interruption UART recv
;   TXIE   = 0 = Pas d'interruption UART trans
;   SSPIE  = 0 = Pas d'interruption Serial port
;   CCP1IE = 0 = Pas d'interruption de CCP1
;   TMR2IE = 0 = Pas d'interruption TMR2-PR2
;   TMR1IE = 0 = Pas d'interruption dpassement de TMR1 

init_it
	movlw   B'10100000'	; Seule interruption autorise = 'TMR0'
	movwf   INTCON		;
	movlw   B'00000000'
	movwf   PIE1		;
	bcf     PIR1,RCIF		; Efface le flag 'Buffer RS232 plein'
	clrf    it_err		; it_err = 0
	clrf    cpt
	clrf    cpt2
	return
	END

;=======================================================================================

; Code annexe pour dbug (copier/coller)

	movlw	.0
	movwf	srv_0
	movlw	.50
	movwf	srv_1
	movlw	.100
	movwf	srv_2
	movlw	.150
	movwf	srv_3
	movlw	.200
	movwf	srv_4

	movlw	.1
	movwf	pwm_r0
	movlw	.2
	movwf	pwm_r1
	movlw	.3
	movwf	pwm_r2
	movlw	.4
	movwf	pwm_r3
	movlw	.5
	movwf	pwm_r4
	movlw	.6
	movwf	pwm_r5
	movlw	.7
	movwf	pwm_r6
	movlw	.8
	movwf	pwm_r7
	movlw	.9
	movwf	pwm_r8
	movlw	.10
	movwf	pwm_r9
	movlw	.11
	movwf	pwm_r10
	movlw	.12
	movwf	pwm_r11
	movlw	.13
	movwf	pwm_r12
	movlw	.61
	movwf	pwm_r12
	movlw	.62
	movwf	pwm_r13
	movlw	.63
	movwf	pwm_r14
	movlw	.64
	movwf	pwm_r15

	movlw	.1
	movwf	out_out3
	movlw	.2
	movwf	out_out4
	movlw	.3
	movwf	out_out5

	movlw	.4
	movwf	out_out21		; nouveau en V3.0
	movlw	.5
	movwf	out_out22
	movlw	.6
	movwf	out_out23
	movlw	.7
	movwf	out_out24
	movlw	.8
	movwf	out_out25
